# coding: latin-1
""" Alle Funktionen, die zu einer Vernderung der Datenbank fhren, stehen
hier. """

import time
import sha
import os
import re
import logging
import shutil
from ebkus.app.ebapi import *
from ebkus.app.ebapih import *
from ebkus.config import config
from ebkus.db.dbapp import cache_on, cache_off, cache_is_on, cache_clear, undo_cached_fields
from ebkus.app.jghdef import get_jghdef


def akteeinf(form):
    """Neue Akte, Fall, Zustaendigkeit"""
    
    akid = check_int_not_empty(form, 'akid', "Aktenid fehlt")
    check_not_exists(akid, Akte,
      "Akte (id: %(id)s, Name: %(na)s, %(vn)s, Geburtsdatum %(gb)s) existiert bereits")
    akte = Akte()
    get_string_fields_new(akte, form, ['na'], check_empty=True)
    get_string_fields_new(akte, form,
                          ['vn', 'ber', 'tl1', 'tl2', 'mail', 'no', 'geschw'],'')
    setAdresse(akte, form)
    akte['gb'] = str(check_date(form, 'gb', 'Fehler im Geburtsdatum'))
    akte['gs'] = check_code(form,'gs', 'gs',
                            "Keine Geschlechtsangabe")
    akte['aufbew'] = check_code(form,'aufbew', 'aufbew',
                            "Keine Aufbewahrungskategorie")
    get_int_fields(akte, form, ['abwlf'])
    abwlf = akte['abwlf']
    if abwlf is not None and abwlf < config.MINIMALE_LOESCHFRIST:
        raise EE("Minimale abweichende Lschfrist ist {} Monate.".format(config.MINIMALE_LOESCHFRIST))
    akte['fs'] = check_code(form,'fs', 'fsfs',
                            "Fehler im Familienstatus", '999')
    akte['stzbg'] = check_code(form, 'stzbg',
                               'stzei', "Kein Stellenzeichen fr die Akte")
    stelle = Code(akte['stzbg'])
    akte['zeit'] = int(time.time())
    
    fall = Fall()
    # Fallbeginn ist identisch mit Zustndigkeitsbeginn
    fall.setDate('bg',
                 check_date(form, 'zubg',
                            "Fehler im Anmeldedatum"))
    if config.ANMELDEPROBLEME_UND_BESCHAEFTIGUNG_BEI_ANMELDUNG_ERFASSEN:
        # qualij auf Schule setzen, falls nicht explizit angegeben und Alter <= 14 Jahre
        if form.get('qualij') == ' ':
            alter = calc_age(akte['gb'], fall.getDate('bg'))
            if alter < 14:
                form['qualij'] = cc('fsqualij', '7') # entfllt, da unter 14
        for feld, kat_code, name in (('qualij', 'fsqualij', 'Beschftigung des/r Jugendlicher/n'),
                                     ('bkm', 'fsbe', 'Beschftigungsverhltnis der Mutter'),
                                     ('bkv', 'fsbe', 'Beschftigungsverhltnis des Vaters'),):
            #print 'FORM', form
            #print 'FELD KAT_CODE NAME', feld, kat_code, name
            akte[feld] = check_code(form, feld,
                                    kat_code,
                                    "Fehlt: %s" % name)
    fall.setDate('zda', Date(0,0,0))
    fall['fn'] = getNewFallnummer(stelle['code'], fall['bgy'])
    fall['status'] = cc('stand', 'l')

    zust = Zustaendigkeit()
    zust['mit_id'] = check_fk(form, 'zumitid', Mitarbeiter,
                              "Kein zustndiger Mitarbeiter fr Fall")
    zust.setDate('bg', fall.getDate('bg'))
    zust.setDate('e', Date(0,0,0))
    
    leist = Leistung()
    leist['mit_id'] = check_fk(form, 'lemitid', Mitarbeiter,
                              "Kein zustndiger Mitarbeiter fr Leistung")
    leist['le'] = check_code(form, 'le', 'fsle', "Keine Leistungsart")
    leist.setDate('bg',
                  check_date(form, 'lebg',
                             "Fehler im Datum fr den Leistungsbeginn"))
    if leist.getDate('bg') < fall.getDate('bg'):
        raise EE("Datum des Leistungsbeginn vor Fallbeginn")
    leist.setDate('e', Date(0,0,0))
    leist['stz'] = check_code(form, 'lestz', 'stzei',
                              "Kein Stellenzeichen fr die Leistung",
                              stelle['id'])

    if config.ANMELDUNGSDATEN_OBLIGATORISCH:
        anmid = check_int_not_empty(form, 'anmid', "Anmeldungsid fehlt")
        check_not_exists(anmid, Anmeldung,
                         "Anmeldung (id: %(id)s, Von: %(von)s) existiert bereits")
        anm = Anmeldung()
        anm['id'] = anmid
        anm = _check_anm(form, anm, None)
    try:
        akte.insert(akid)
        fall['akte_id'] = akte['id']
        fall.new()
        fall.insert()
        if config.ANMELDUNGSDATEN_OBLIGATORISCH:
            anm['fall_id'] = fall['id']
            anm.insert()
        zust['fall_id'] = fall['id']
        zust.new()
        zust.insert()
        leist['fall_id'] = fall['id']
        leist.new()
        leist.insert()
    except Exception, args:
        try: akte.delete()
        except: pass
        try: fall.delete()
        except: pass
        try: zust.delete()
        except: pass
        try: leist.delete()
        except: pass
        raise EBUpdateError("Fehler beim Einfgen in die Datenbank: %s" % str(args))
        
def updakte(form):

    akteold = check_exists(form, 'akid', Akte, "Aktenid fehlt")
    fall = akteold['aktueller_fall']
    akte = Akte()
    get_string_fields_new(akte, form, ['na'], akteold, check_empty=True)
    akte['gs'] = check_code(form,'gs', 'gs',
                            "Keine Geschlechtsangabe")
    akte['aufbew'] = check_code(form,'aufbew', 'aufbew',
                            "Keine Aufbewahrungskategorie")
    get_int_fields(akte, form, ['abwlf'])
    abwlf = akte['abwlf']
    if abwlf is not None and abwlf < config.MINIMALE_LOESCHFRIST:
        raise EE("Minimale abweichende Lschfrist ist {} Monate.".format(config.MINIMALE_LOESCHFRIST))
    akte['gb'] = str(check_date(form, 'gb', 'Fehler im Geburtsdatum'))
    get_string_fields_new(akte, form,
                          ['vn', 'ber', 
                           'tl1', 'tl2', 'mail', 'no', 'geschw'], akteold)
    if fall and config.ANMELDEPROBLEME_UND_BESCHAEFTIGUNG_BEI_ANMELDUNG_ERFASSEN:
        # qualij auf Schule setzen, falls nicht explizit angegeben und Alter <= 14 Jahre
        if form.get('qualij') == ' ':
            alter = calc_age(akte['gb'], fall.getDate('bg'))
            if alter < 14:
                form['qualij'] = cc('fsqualij', '7') # entfllt, da unter 14
        for feld, kat_code, name in (('qualij', 'fsqualij', 'Beschftigung des/r Jugendlicher/n'),
                                     ('bkm', 'fsbe', 'Beschftigungsverhltnis der Mutter'),
                                     ('bkv', 'fsbe', 'Beschftigungsverhltnis des Vaters'),):
            #print 'FORM', form
            #print 'FELD KAT_CODE NAME', feld, kat_code, name
            akte[feld] = check_code(form, feld,
                                    kat_code,
                                    "Fehlt: %s" % name, akteold)
    
    setAdresse(akte, form)
    akte['fs'] = check_code(form,'fs', 'fsfs',
                            "Fehler im Familienstatus", akteold)
    # Werte erst dann bernehmen, wenn keine Fehler aufgetreten sind
    akteold.update(akte)
    _stamp_akte(akteold)
    
def perseinf(form):
    """Neue Bezugsperson."""
    
    bpid = check_int_not_empty(form, 'bpid', "Bezugspersonenid fehlt")
    check_not_exists(bpid, Bezugsperson,
      "Bezugsperson (id: %(id)s, Name: %(na)s, %(vn)s Geburtsdatum %(gb)s) existiert bereits")
    pers = Bezugsperson()
    pers['akte_id'] = check_fk(form, 'akid', Akte, "Keine Akte")
    get_string_fields_new(pers, form,
                          ['vn', 'na', 'ber', 'tl1', 'tl2', 'mail', 'no'],'')
    if pers['vn'] == '' and pers['na'] == '':
        raise EE("Kein Name")
    pers['verw'] = check_code(form, 'verw', 'klerv',
                              "Fehler im Verwandtschaftsgrad", '999')
    gs = form.get('gs')
    if gs and not gs == ' ':
        gs = check_code(form,'gs', 'gs',
                        "Keine Geschlechtsangabe", '')
    else:
        verwname = Code(pers['verw'])['name'].lower()
        if 'vater' in verwname:
            gs = cc('gs', '1')
        elif 'mutter' in verwname:
            gs = cc('gs', '2')
        else:
            gs = None
    pers['gs'] = gs
    gb = check_date(form, 'gb', 'Fehler im Geburtsdatum', (0,0,0),
                    alle_jahrgaenge_akzeptieren=True)
    pers['gb'] = gb.year != 0 and str(gb) or ''
    setAdresse(pers, form)
    pers['fs'] = check_code(form,'fs', 'fsfs',
                            "Fehler im Familienstatus", '999')
    pers['nobed'] = check_code(form, 'nobed', 'notizbed',
                               "Fehler in Notizbedeutung", cc('notizbed', 'f'))
    pers['vrt'] = check_code(form, 'vrt', 'vert',
                             "Fehler in Verteiler", 'f')
    try:
        pers.insert(bpid)
    except Exception, args:
        try: pers.delete()
        except: pass
        raise EBUpdateError("Fehler beim Einfgen in die Datenbank: %s" % str(args))
    _stamp_akte(pers['akte'])
    
    
def updpers(form):
    """Update der Bezugsperson."""
    
##     # unsystematische HACK!!
##     fs = form.get('nobed')
##     if not fs:
##         form['nobed'] = cc('notizbed', 'f')
        
    persold = check_exists(form, 'bpid', Bezugsperson, "Bezugspersonid fehlt")
    pers = Bezugsperson()
    get_string_fields_new(pers, form,
                      ['vn', 'na', 'ber', 'tl1', 'tl2', 'mail', 'no','hsnr'],
                      persold)
    if pers['vn'] == '' and pers['na'] == '':
        raise EE("Kein Name")
    pers['verw'] = check_code(form, 'verw', 'klerv',
                              "Fehler im Verwandtschaftsgrad", persold)
    gs = form.get('gs').strip()
    if gs:
        gs = check_code(form,'gs', 'gs',
                        "Keine Geschlechtsangabe", '')
    else:
        verwname = Code(pers['verw'])['name'].lower()
        if 'vater' in verwname:
            gs = cc('gs', '1')
        elif 'mutter' in verwname:
            gs = cc('gs', '2')
        else:
            gs = None
    pers['gs'] = gs
    gb = check_date(form, 'gb', 'Fehler im Geburtsdatum', (0,0,0),
                    alle_jahrgaenge_akzeptieren=True)
    pers['gb'] = gb.year != 0 and str(gb) or ''
    setAdresse(pers, form)
    pers['fs'] = check_code(form,'fs', 'fsfs',
                            "Fehler im Familienstatus", persold)
    pers['nobed'] = check_code(form, 'nobed', 'notizbed',
                               "Fehler in Notizbedeutung", cc('notizbed', 'f'))
    pers['vrt'] = check_code(form, 'vrt', 'vert',
                             "Fehler in Verteiler", persold)
    
    persold.update(pers)
    _stamp_akte(persold['akte'])
    

def removepers(form):
    pers = check_exists(form, 'bpid', Bezugsperson, "Bezugspersonid fehlt")
    if pers['gruppen']:
        raise EE('Mitglied einer Gruppe kann nicht gelscht werden')
    if pers['fallberatungskontakte']:
        raise EE('Teilnehmer/in an Beratungskontakt kann nicht gelscht werden')
    pers.delete()

    
def einreinf(form):
    """Neuer Einrichtungskontakt."""
    
    einrid = check_int_not_empty(form, 'einrid', "Einrichtungskontaktid fehlt")
    check_not_exists(einrid, Einrichtungskontakt,
      "Einrichtungskontakt (id: %(id)s, Name: %(na)s) existiert bereits")
    einr = Einrichtungskontakt()
    einr['akte_id'] = check_fk(form, 'akid', Akte, "Keine Akte")
    get_string_fields_new(einr, form, ['na'], check_empty=True)
    get_string_fields_new(einr, form, ['tl1','tl2', 'mail', 'no'],'')
    einr['insta'] = check_code(form, 'insta', 'klinsta',
                               "Fehler in Institution", '999')
    einr['nobed'] = check_code(form, 'nobed', 'notizbed',
                               "Fehler in Notizbedeutung", cc('notizbed', 'f'))
    einr['status'] = check_code(form, 'status', 'einrstat',
                                "Fehler in Einrichtungsstatus", cc('einrstat', 'nein'))
    
    einr.insert(einrid)
    _stamp_akte(einr['akte'])
    
    
def updeinr(form):
    """Update eines Einrichtungskontaktes."""
    
##     # unsystematische HACK!!
##     fs = form.get('nobed')
##     if not fs:
##         form['nobed'] = cc('notizbed', 'f')
        
    einrold = check_exists(form, 'einrid', Einrichtungskontakt, "Einrichtungskontaktid fehlt")
    einr = Einrichtungskontakt()
    get_string_fields_new(einr, form, ['na','tl1','tl2', 'mail', 'no'], einrold)
    if einr['na'] == '':
        raise EE("Kein Name")
    einr['insta'] = check_code(form, 'insta', 'klinsta',
                               "Fehler in Institution", einrold)
    einr['nobed'] = check_code(form, 'nobed', 'notizbed',
                               "Fehler in Notizbedeutung", cc('notizbed', 'f'))
    einr['status'] = check_code(form, 'status', 'einrstat',
                                "Fehler in Einrichtungsstatus", cc('einrstat', 'nein'))
    
    einrold.update(einr)
    _stamp_akte(einrold['akte'])
    

def removeeinr(form):
    einr = check_exists(form, 'einrid', Einrichtungskontakt, "Einrichtungskontaktid fehlt")
    einr.delete()


def _check_anmeinf(form):
    # Nur fr Einfhrung (siehe akteeinf, waufneinf)
    anmid = check_int_not_empty(form, 'anmid', "Anmeldungsid fehlt")
    check_not_exists(anmid, Anmeldung,
      "Anmeldung (id: %(id)s, Von: %(von)s) existiert bereits")
    anm = Anmeldung()
    anm['id'] = anmid
    anm = _check_anm(form, anm, None)
    return anm

def _check_anm(form, anm, anmold):
    # Gemeinsam fr Einfhrung und Update
    get_string_fields_new(anm, form, ['von'], anmold, check_empty=True)
    get_string_fields_new(anm, form, ['mtl','me', 'mg', ('anm_no', 'no')], anmold)
    anm['zm'] = check_code(form, 'zm', 'fszm', "Fehler im Zugangsmodus", anmold)
    if config.ANMELDEPROBLEME_UND_BESCHAEFTIGUNG_BEI_ANMELDUNG_ERFASSEN:
        anm['anmprobleme'] = check_multi_code(form, 'anmprobleme',
                                              'fsba',
                                              "Fehlt: Anmeldeprobleme")
    return anm

def anmeinf(form):
    """Neue Anmeldung."""
    # anmid = check_int_not_empty(form, 'anmid', "Anmeldungsid fehlt")
    # check_not_exists(anmid, Anmeldung,
    #   "Anmeldung (id: %(id)s, Von: %(von)s) existiert bereits")
    # anm = Anmeldung()
    # anm['id'] = anmid
    # #anm = _check_anmeinf(form) #old

    anm = _check_anmeinf(form)

    anm['fall_id'] = check_fk(form, 'fallid', Fall, "Kein Fall")
    fall = Fall(anm['fall_id'])
    vorhandene_anmeldung = fall['anmeldung']
    if len(vorhandene_anmeldung) > 0:
        raise EE("Anmeldung fr Fall %(fn)s schon vorhanden" % fall)
    try:
        anm.insert()
    except:
        try: anm.delete()
        except: pass
    _stamp_akte(anm['akte'])
    
    
def updanm(form):
    """Update der Anmeldung."""
    anmold = check_exists(form, 'anmid', Anmeldung, "Keine Anmeldungsid")

    anm = Anmeldung()
    anm = _check_anm(form, anm, anmold)
    anmold.update(anm)
    _stamp_akte(anmold['akte'])
    
def leisteinf(form):
    """Neue Leistung."""
    
    leistid = check_int_not_empty(form, 'leistid', "Leistungsid fehlt")
    check_not_exists(leistid, Leistung,
      "Leistung (id: %(id)s) existiert bereits")
    leist = Leistung()
    leist['fall_id'] = check_fk(form, 'fallid', Fall, "Kein Fall")
    leist['mit_id'] = check_fk(form, 'mitid', Mitarbeiter, "Kein Mitarbeiter")
    leist['le'] = check_code(form, 'le', 'fsle', "Fehler in Leistungsart")
    
    
    beginndatum = check_date(form, 'bg', "Fehler im Leistungsbeginndatum")
    endedatum = check_date(form, 'e', "Fehler im Leistungsendedatum", Date(0,0,0),
                           maybezero = 1,
                           maybefuture = config.LEISTUNGSENDE_IN_ZUKUNFT_ERLAUBT and 1 or 0,
                           )
    
    if endedatum < beginndatum:
        raise EE("Endedatum vor Beginndatum")
    fall = Fall(leist['fall_id'])
    if beginndatum < fall.getDate('bg'):
        raise EE("Beginndatum vor Fallbeginn")
        
    leist.setDate('bg', beginndatum)
    leist.setDate('e', endedatum)
    
    
    leist['stz'] = check_code(form, 'stz', 'stzei',
                              "Kein Stellenzeichen fr die Leistung")
    try:
        leist.insert(leistid)
    except:
        try: leist.delete()
        except: pass
    _stamp_akte(leist['akte'])
    
    
def updleist(form):
    """Update der Leistung."""
    
    leistold = check_exists(form,'leistid', Leistung, "Keine Leistungsid")
    fall = leistold['fall']
    
    leist = Leistung()
    
    leist['mit_id'] = check_fk(form, 'mitid', Mitarbeiter, "Fehler in Mitarbeiter", leistold)
    leist['le'] = check_code(form, 'le', 'fsle', "Fehler in Leistungsart", leistold)
    
    beginndatum = check_date(form, 'bg', "Fehler im Leistungsbeginndatum", leistold)
    endedatum = check_date(form, 'e', "Fehler im Leistungsendedatum", Date(0,0,0),
                           maybezero = 1,
                           maybefuture = config.LEISTUNGSENDE_IN_ZUKUNFT_ERLAUBT and 1 or 0,
                           )
    # berprfungen?
    if endedatum < beginndatum:
        raise EE("Endedatum vor Beginndatum")
        
    if beginndatum < fall.getDate('bg'):
        raise EE("Beginndatum vor Fallbeginn")
        
    leist.setDate('bg', beginndatum)
    leist.setDate('e', endedatum)
    
    leist['stz'] = check_code(form, 'stz', 'stzei',
                              "Kein Stellenzeichen fr die Leistung",
                              leistold)
    leistold.update(leist)
    _stamp_akte(leistold['akte'])


def removeleist(form):
    """Lschen der Leistung."""
    leist = check_exists(form,'leistid', Leistung, "Keine Leistungsid")
    if len(leist['fall']['leistungen']) < 2:
        raise EE('Es muss immer mindestens eine Leistung geben')
    if leist['beratungskontakte']:
        raise EE('Leistung in einem Beratungskontakt kann nicht gelscht werden')
    leist.delete()
        
def _bkont_check(form, bkont):
    #print '_bkont_bs_check', form
    fall_id = check_fk(form, 'fallid', Fall, "Kein Fall")
    fall_ids = check_list(form, 'bkfallid', 'Keine Flle')
    mit_ids = check_list(form, 'mitid', 'Keine Mitarbeiter')
    fall = Fall(fall_id)
    faelle = [Fall(id) for id in fall_ids]
    if fall not in faelle:
        raise EE("Der Klient der aktuell geffneten Klientenkarte muss am Beratungskontakt beteiligt sein. ")
    mitarbeiter = [Mitarbeiter(id) for id in mit_ids]
    datum = check_date(form, 'k', "Fehler im Beratungskontaktdatum")
    for f in faelle:
        if datum < f.getDate('bg'):
            raise EE('Kontaktdatum liegt vor Fallbeginn eines teilnehmenden Klienten: %s (Fallbeginn: %s)' % (f['akte__name'], f.getDate('bg')))
    if config.BERATUNGSKONTAKTE_BS:
        # frueheres Quartal?
        fruehquart = form.get('fruehquart')
        mdiff = datum.diff(today()) # Differenz, nur die Monate
        if fruehquart != '1' and (mdiff >= 3 or datum.quartal() != today().quartal()):
            # 'frheres Quartal' nicht markiert
            # Datum des Beratungskontakts liegt in einem frheren Quartal
            raise EE("Datum des Beratungskontakts liegt in einem frheren Quartal. Wenn trotzdem gespeichert werden soll, bitte 'frheres Quartal' markieren.")
        zeit = check_time(form, 'k', "Fehler in Uhrzeit", Time())
        bkont.setTime('k', zeit)
        bkont['art_bs'] = check_code(form, 'art_bs', 'kabs', "Fehler in Beratungskontaktart")
        mc = check_multi_code(form, 'teilnehmer_bs', 'teilnbs',
                              "Teilnehmerangabe fehlt",
                              # default=[Code(kat_code='teilnbs',
                              #               code='0')],
                              )
        #print 'MULTICODE', mc, type(mc)
        bkont['teilnehmer_bs'] = mc
        bkont['offenespr'] = check_code(form, 'offenespr', 'ja_nein', "", cc('ja_nein', 2))
        if Code(bkont['art_bs'])['code'] == '5':
            # ausgefallener Kontakt automatisch 20 Minuten
            bkont['dauer'] = 20
            bkont['anzahl'] = 0
        else:
            bkont['dauer'] = check_int_not_empty(form, 'dauer', "Keine Dauer angegeben")
            bkont['anzahl'] = check_int_not_empty(form, 'anzahl', "Fehler in Anzahl der Teilnehmer", 1)
        if not (bkont['dauer'] % 10) == 0:
            raise EE("Bitte Kontaktdauer nur in 10-er Schritten angeben, z.B. 20, 30, 60.")
    else:
        bkont['art'] = check_code(form, 'art', 'fska', "Fehler in Beratungskontaktart")
        dauer_kat = check_code(form, 'dauer_kat', 'fskd', "Fehler in Kontaktdauer")
        # wir tragen einfach den Mittelwert zwischen den Bereichsgrenzen ein
        bkont['dauer'] = int((Code(dauer_kat)['mini'] + Code(dauer_kat)['maxi'])/2.)
    # try:
    #     # erstgespraech ist nur dann nicht None, wenn Beratungsdokumentation gemacht wurde/wird.
    #     bkont['erstgespraech'] = check_code(form, 'erstgespraech', 'ja_nein', "Kein Erstgespraech")
    #     BERATUNG_DOKUMENTATION = True
    # except EE:
    #     # keine Beratungsdokumentation fr diesen Fall
    #     BERATUNG_DOKUMENTATION = False
    if config.BERATUNGSKONTAKT_DOKUMENTATION:
        bkont['erstgespraech'] = check_code(form, 'erstgespraech', 'ja_nein', "Kein Erstgespraech")
        is_erstgespraech = (bkont['erstgespraech'] == cc('ja_nein', '1'))
        if config.BERATUNGSKONTAKTE_BS and is_erstgespraech:
            if not bkont['art_bs'] == cc('kabs', '1'):
                raise EE('Erstgesprch muss persnlicher Kontakt sein')
            if len(faelle) > 1:
                raise EE('Mehr als ein Klient beim Erstgesprch')
        # Das Datum eines Beratungskontakts sollte immer nach dem Erstgesprch eines Teilnehmers sein.
        # Wenn ein Teilnehmer kein Erstgesprch hatte (Fall begann vor Dokumentationsbeginn zB)
        # spielt das keine Rolle.
        if config.BERATUNGSKONTAKTE_BS: # sonst gibt es kein Erstgesprch
            for f in faelle:
                for bk in f['beratungskontakte']:
                    if bk['erstgespraech'] == cc('ja_nein', 1):
                        if datum < bk.getDate('k'):
                            raise EE('Kontaktdatum liegt vor dem Datum des Erstgesprchs eines teilnehmenden Klienten: %s (Erstgesprch am: %s)' % (f['akte__name'], bk.getDate('k')))
        try:
            bkont['pervid'] = check_code(form, 'pervid', 'ja_nein', "Kein per Video")
        except EE:
            bkont['pervid'] = None
        try:
            bkont['aufklaerung'] = check_code(form, 'aufklaerung', 'ja_nein', 'Keine Angabe zur Aufklrung')
        except EE:
            # Kann jetzt auch beim Erstgesprch offen bleiben.
            # Item wird bei Folgekontakten immer wieder vorgelegt,
            # bis ein Hckchen gemacht wird.
            # Damit man das Hkchen wieder entfernen kann
            bkont['aufklaerung'] = None
        if config.BERATUNGSKONTAKTE_BS and bkont['art_bs'] == cc('kabs', 5): 
            # ausgefallener Kontakt, inhalt nicht Pflicht
            check_empty = False
        else:
            # sonst ist inhalt Pflicht
            check_empty = True
        get_string_fields_new(bkont, form, ['inhalt'], check_empty=check_empty)
        # Optional
        get_string_fields_new(bkont, form,
                              ['vorgehen', 'diagnose', 'kontrakt', 'richtung', 'verlauf', 'vereinbarung'],
                              default='')
        try:
            # termin ist kein code sondern handgestrickter INT
            bkont['termin'] = check_int_not_empty(form, 'termin', 'Keine Angabe zur Terminabsprache')
        except EE:
            # Bei persnlichen Kontakten ist Terminabsprache Pflicht
            if config.BERATUNGSKONTAKTE_BS and bkont['art_bs'] == cc('kabs', '1'):
                raise
            else:
                bkont['termin'] = None
        bkont['fall8a'] = check_code(form, 'fall8a', 'ja_nein', 'Kein Fall8a', '2')
        if bkont.getDate('d') < bkont.getDate('k'):
            raise EE("Datum des Erstellens der Beratungsdokumentation kann nicht vor dem Kontaktdatum liegen.")
    get_string_fields_new(bkont, form, ['no'], '')
    for fall in faelle:
        if datum < fall.getDate('bg'):
            raise EE("Datum vor Fallbeginn von %(akte__vn)s %(akte__na)s" % fall )
        if fall.get('ey') and datum > fall.getDate('e'):
            raise EE("Datum nach Fallende von %(akte__vn)s %(akte__na)s" % fall)
    bkont.setDate('k', datum)
    bkont['stz'] = check_code(form, 'stz', 'stzei',
                              "Kein Stellenzeichen fr den Beratungskontakt")
    return mitarbeiter, faelle

def _bkont_upd_mitarbeiter_faelle(bkont, mitarbeiter, faelle):
    def compare(new_list, old_list):
        todel = []
        toins = []
        for x in new_list:
            if x not in old_list:
                toins.append(x)
        for x in old_list:
            if x not in new_list:
                todel.append(x)
        return todel, toins
    mbl = MitarbeiterberatungskontaktList(where='bkont_id=%(id)s' % bkont)
    todel, toins = compare(mitarbeiter, mbl)
    for m in todel:
        m.delete()
    for m in toins:
        mb = Mitarbeiterberatungskontakt()
        mb.init(
            mit_id=m['id'],
            bkont_id=bkont['id'],
            )
        mb.new()
        mb.insert()
    fbl = FallberatungskontaktList(where='bkont_id=%(id)s' % bkont)
    todel, toins = compare(faelle, fbl)
    for x in todel:
        x.delete()
    for f in toins:
        fb = Fallberatungskontakt()
        fb.init(
            fall_id=f['id'],
            bkont_id=bkont['id'],
            )
        fb.new()
        fb.insert()

## def bkonteinf(form):
##     """Neuer Beratungskontakt."""
##     if not config.BERATUNGSKONTAKTE:
##         raise EBUpdateError("Aufruf von bkonteinf ohne config.BERATUNGSKONTAKTE")
##     bkontid = check_int_not_empty(form, 'bkontid', "Beratungskontaktid fehlt")
##     check_not_exists(bkontid, Beratungskontakt,
##                      "Beratungskontakt (id: %(id)s) existiert bereits")
##     bkont = Beratungskontakt()
##     _bkont_check(form, bkont)
##     try:
##         bkont.insert(bkontid)
##     except:
##         try: bkont.delete()
##         except: pass
##     _stamp_akte(bkont['fall__akte'])

def bkonteinf(form):
    """Neuer Beratungskontakt."""
    bkontid = check_int_not_empty(form, 'bkontid', "Beratungskontaktid fehlt")
    bearbeiter_id = check_int_not_empty(form, 'bearbeiterid', "Fehler in Bearbeiter")
    check_not_exists(bkontid, Beratungskontakt,
                     "Beratungskontakt (id: %(id)s) existiert bereits")
    bkont = Beratungskontakt()
    bkont.setDate('d', today())
    bkont['dvon_id'] = bearbeiter_id
    mitarbeiter, faelle = _bkont_check(form, bkont)
    try:
        bkont.insert(bkontid)
        _bkont_upd_mitarbeiter_faelle(bkont, mitarbeiter, faelle)
    except:
        # das fhrt dazu, dass bkonts sang- und klanglos verloren gehen!!
        try: bkont.delete()
        except: pass
    for f in bkont['faelle']:
        _stamp_akte(f['akte'])
    
## def updbkont(form):
##     """Update des Beratungskontakts."""
##     if not config.BERATUNGSKONTAKTE:
##         raise EBUpdateError("Aufruf von updbkont ohne config.BERATUNGSKONTAKTE")
##     bkontold = check_exists(form,'bkontid', Beratungskontakt, "Keine Beratungskontaktid")
##     bkont = Beratungskontakt()
##     _bkont_check(form, bkont)
##     bkontold.update(bkont)
##     _stamp_akte(bkontold['fall__akte'])
    
def updbkont(form):
    """Update des Beratungskontakts."""
    bkontold = check_exists(form,'bkontid', Beratungskontakt, "Keine Beratungskontaktid")
    bearbeiter_id = check_int_not_empty(form, 'bearbeiterid', "Fehler in Bearbeiter")
    bkont = Beratungskontakt()
    if config.BERATUNGSKONTAKT_DOKUMENTATION:
        # evt. wird bestehender Kontakt zum ersten Mal mit config.BERATUNGSKONTAKT_DOKUMENTATION
        # bearbeitet. Dann today() als Erstellungsdatum, das sonst in bkonteinf gesetzt wird.
        if not bkontold['dy']:
            bkont.setDate('d', today())
        if not bkontold['dvon_id']:
            # alter Beratungskontakt, sonst gbe es dvon_id, siehe bkonteinf
            # Zustndigen nehmen als Dokuersteller (nicht immer richtig, aber meistens)
            fall_id = check_int_not_empty(form, 'fallid', "Fehler in Fall")
            bkont['dvon_id'] = Fall(fall_id)['zustaendig__mit__id']
        bkont.setDate('dlc', today())
        bkont['dlcvon_id'] = bearbeiter_id
    if not bkontold['noch_modifizierbar']:
        raise EE("Beratungskontakt kann nach {} Werktagen nicht mehr bearbeitet werden."
                 .format(config.BERATUNGSKONTAKT_DOKUMENTATION_BEARBEITUNGS_FRIST))

    # if (config.BERATUNGSKONTAKT_DOKUMENTATION and 
    #     bkontold.getDate('d').diff_weekdays(bkont.getDate('dlc')) > config.BERATUNGSKONTAKT_DOKUMENTATION_BEARBEITUNGS_FRIST):
    #     raise EE("Beratungskontakt kann nach {} Werktagen nicht mehr bearbeitet werden."
    #              .format(config.BERATUNGSKONTAKT_DOKUMENTATION_BEARBEITUNGS_FRIST))

    mitarbeiter, faelle = _bkont_check(form, bkont)
    bkontold.update(bkont)
    _bkont_upd_mitarbeiter_faelle(bkontold, mitarbeiter, faelle)
    for f in bkontold['faelle']:
        _stamp_akte(f['akte'])
    

def removebkont(form):
    bkont = check_exists(form,'bkontid', Beratungskontakt, "Keine Beratungskontaktid")
    faelle = bkont['faelle']
    MitarbeiterberatungskontaktList(where='bkont_id=%(id)s' % bkont).deleteall()
    FallberatungskontaktList(where='bkont_id=%(id)s' % bkont).deleteall()
    bkont.delete()
    for f in faelle:
        _stamp_akte(f['akte'])
    
# Fallunabhngige Aktivitten Braunschweig
def _fua_bs_check(form, fua):
    #print '_fua_bs_check', form
    #fua['mit_id'] = check_fk(form, 'mitid', Mitarbeiter, "Kein Mitarbeiter")
    fua['mit_id'] = None
    fua['art'] = check_code(form, 'art', 'fuabs', "Fehler in Aktivittstart")
    get_string_fields_new(fua, form, ['no'], '')
    if fua['art'] == cc('fuabs', '1'):
        fua['dauer'] = 20
    else:
        fua['dauer'] = check_int_not_empty(form, 'dauer', "Fehler in Dauer")
        if not (fua['dauer'] % 10) == 0:
            raise EE("Bitte Aktivittsdauer nur in 10-er Schritten angeben, z.B. 20, 30, 60, etc.")
    datum = check_date(form, 'k', "Fehler im Aktivittsdatum")
    fua.setDate('k', datum)
    fua['stz'] = check_code(form, 'stz', 'stzei',
                              "Kein Stellenzeichen fr die Aktivitt")
def _fua_bs_upd_mitarbeiter(fua_bs, mit_ids):
    Mitarbeiterfua_bsList(where='fua_bs_id=%(id)s' % fua_bs).deleteall()
    for mit_id in mit_ids:
        mb = Mitarbeiterfua_bs()
        mb.init(
            mit_id=int(mit_id),
            fua_bs_id=fua_bs['id'],
            )
        mb.new()
        mb.insert()

def fuabseinf(form):
    """Neue Aktivitt."""
    if not config.FALLUNABHAENGIGE_AKTIVITAETEN_BS:
        raise EBUpdateError("Aufruf von fuabseinf ohne config.FALLUNABHAENGIGE_AKTIVITAETEN_BS")
    fuaid = check_int_not_empty(form, 'fuaid', "Aktivitts-ID fehlt")
    check_not_exists(fuaid, Fua_BS,
                     "Aktivitt (id: %(id)s) existiert bereits")
    mit_ids = check_list(form, 'mitid', 'Keine Mitarbeiter')
    fua = Fua_BS()
    _fua_bs_check(form, fua)
    try:
        fua.insert(fuaid)
        _fua_bs_upd_mitarbeiter(fua, mit_ids)
    except:
        raise
        try: fua.delete()
        except: pass
    
def updfuabs(form):
    """Update der fallunabhngigen Aktivitt."""
    if not config.FALLUNABHAENGIGE_AKTIVITAETEN_BS:
        raise EBUpdateError("Aufruf von updfuabs ohne config.FALLUNABHAENGIGE_AKTIVITAETEN_BS")
    fuaold = check_exists(form,'fuaid', Fua_BS, "Keine Aktivitts-ID")
    mit_ids = check_list(form, 'mitid', 'Keine Mitarbeiter')
    fua = Fua_BS()
    _fua_bs_check(form, fua)
    fuaold.update(fua)
    _fua_bs_upd_mitarbeiter(fuaold, mit_ids)

def removefuabs(form):
    if not config.FALLUNABHAENGIGE_AKTIVITAETEN_BS:
        raise EBUpdateError("Aufruf von updfuabs ohne config.FALLUNABHAENGIGE_AKTIVITAETEN_BS")
    fua = check_exists(form,'fuaid', Fua_BS, "Keine Aktivitts-ID")
    _fua_bs_upd_mitarbeiter(fua, [])
    fua.delete()
    
def _zust_upd_fachstat(zust):
    """Den Mitarbeiter der Zustndigkeit als Mitarbeiter in die zum Fall 
    gehrende Fachstatistik eintragen, falls vorhanden.
    """
    fall = zust['fall']
    fachstatistiken = fall['fachstatistiken']
    if fachstatistiken:
        fs = fachstatistiken[0]
        fs.update({'mit_id': zust['mit_id']})

def zusteinf(form):
    """Neue Zustndigkeit."""

    # nicht am selben Tag beginnen wie der vorige
    zustid = check_int_not_empty(form, 'zustid', "Zustndigkeitsid fehlt")
    check_not_exists(zustid, Zustaendigkeit,
      "Zustndigkeit (id: %(id)s) existiert bereits")
    zust = Zustaendigkeit()
    zust['fall_id'] = check_fk(form, 'fallid', Fall, "Kein Fall")
    zust['mit_id'] = check_fk(form, 'mitid', Mitarbeiter, "Kein Mitarbeiter")
    beginndatum =  check_date(form, 'bg',
                                "Fehler im Zustndigkeitsbeginndatum")
    zust.setDate('bg',
                 beginndatum)
    # Kann 0 sein, 0 als Default
    endedatum = check_date(form, 'e',
                           "Fehler im Zustndigkeitsendedatum", Date(0,0,0),
                           maybezero = 1)
    zust.setDate('e', endedatum)
    # if endedatum < beginndatum:
    #     raise EE("Zustndigkeitsende vor Beginn")
    # Zustndigkeiten mit Beginn == Ende nicht mehr zulssig
    if endedatum <= beginndatum:
        raise EE("Zustndigkeitsende muss mindestens ein Tag nach dem Beginn sein")
    aktzustold = check_exists(form, 'aktuellzustid' ,
                              Zustaendigkeit, "Kein aktueller Zustand")
    aktzust_endedatum = aktzustold.getDate('e')
    if beginndatum < aktzustold.getDate('bg'):
        raise EE("Zustndigkeitsbeginn vor Beginn der aktuellen Zustndigkeit")
    aktzust = Zustaendigkeit()
    aktzust.setDate('e', beginndatum)
    try:
        # Mitarbeiter ist jetzt direkt zustndig, keine zusaetzliche Berechtigung mehr ntig
        z = ZusaetzlicheBerechtigung(mit_id=zust['mit_id'], fall_id=zust['fall_id'])
        z.delete()
    except Exception:
        pass # Mitarbeiter hat keine zustzliche Berechtigung fr diesen Fall
    try:
        zust.insert(zustid)
        aktzustold.update(aktzust)
        _zust_upd_fachstat(zust)
    except:
        try: zust.delete()
        except: pass
        
    _stamp_akte(zust['akte'])
    
    ##
    ## Es ist immer genau ein Mitarbeiter pro Zeiteinheit zustaendig.
    ## 1. Ueberschneidungen (groesser 1 Tag, weil Ende u. Beginndatum
    ## identisch ist) Fehler ebenso wenn 2 Mitarbeiter gleichzeitig
    ##  aktuell zustaendig sind.
    ## 3. Aelteste Zustaendigkeit sollte mit Fallbeginn identisch sein
    ##    (Rueckmelden).
    


    
def updzust(form):
    """Update der Zustndigkeit."""
    
    zustold  = check_exists(form, 'zustid', Zustaendigkeit,
                            "Keine Zustaendigksid")
    # Sollte heute nicht mehr mglich sein, aber es gibt alte Daten,
    # wo Zustndigkeit am selben Tag beginnt und endet.
    zustold_bg_e_gleich = (zustold.getDate('bg') == zustold.getDate('e'))
    prevold = zustold['prev']
    nextold = zustold['next']
    fall = zustold['fall']
    zust = Zustaendigkeit()
    prev = Zustaendigkeit()
    next = Zustaendigkeit()
    # Soll tatschlich der Mitarbeiter fr eine Zustndigkeit wechseln knnen?
    zust['mit_id'] = check_fk(form, 'mitid', Mitarbeiter, "Fehler in Mitarbeiter", zustold)
    mitarbeiter_change = (zust['mit_id'] != zustold['mit_id'])
    
    beginndatum = check_date(form, 'bg', "Fehler im Beginndatum", zustold)
    if prevold:
        if beginndatum <= prevold.getDate('bg'):
            raise EE('Beginndatum muss nach dem Beginn der letzten Zustndigkeit sein')
    else:
        # erste Zustndigkeit
        if beginndatum != fall.getDate('bg'):
            raise EE('Erste Zustndigkeit muss mit dem Fall beginnen')
    prev.setDate('e', beginndatum)
    zust.setDate('bg', beginndatum)
    endedatum = check_date(form, 'e', "Fehler im Enddatum",
                           zustold, maybezero = 1)
    datums_aenderung = (beginndatum != zustold.getDate('bg') or
                        endedatum   != zustold.getDate('e'))
    # Wenn Beginn oder Ende gendert wurden, keine Zustndigkeiten mehr
    # zulassen, wo Beginn == Ende.
    # Vorhandene Eintrge mit Beginn == Ende drfen aber bleiben, und es muss
    # auch der Mitarbeiter gendert werden knnen.
    if datums_aenderung:
        if endedatum <= beginndatum:
            raise EE("Zustndigkeitsende muss mindestens ein Tag nach dem Beginn sein")
    if nextold:
        if endedatum >= nextold.getDate('e'):
            raise EE('Endedatum muss vor dem Ende der folgenden Zustndigkeit sein')
    else:
        # erste Zustndigkeit
        if endedatum != fall.getDate('zda'):
            raise EE('Letzte Zustndigkeit muss mit dem Fall enden')
    next.setDate('bg', endedatum)
    zust.setDate('e', endedatum)
    zustold.update(zust)
    if prevold:
        prevold.update(prev)
    if nextold:
        nextold.update(next)
    if mitarbeiter_change and fall['zuletzt_zustaendig'] == zustold:
        _zust_upd_fachstat(zustold)
    aktuelle_zustaendigkeit = fall['zustaendig']
    if aktuelle_zustaendigkeit:
        try:
            # Mitarbeiter ist jetzt direkt zustndig, keine zusaetzliche Berechtigung mehr ntig
            z = ZusaetzlicheBerechtigung(mit_id=zustold['mit_id'], fall_id=zustold['fall_id'])
            z.delete()
        except Exception:
            pass # Mitarbeiter hat keine zustzliche Berechtigung fr diesen Fall
    _stamp_akte(zustold['akte'])
    
    
def dokeinf(form):
    """Neues Dokument."""
    
    import os
    
    dokid = check_int_not_empty(form, 'dokid', "dokid fehlt")
    check_not_exists(dokid, Dokument,
      "Dokument-ID (id: %(id)s) existiert bereits")
    dok = Dokument()
    dok['fall_id'] = check_fk(form, 'fallid', Fall, "Kein Fall")
    fall = dok['fall']
    akteold = fall['akte']
    dok['mit_id'] = check_fk(form, 'mitid', Mitarbeiter, "Kein Mitarbeiter")
    get_string_fields_new(dok, form, ['betr'], check_empty=True)
    dok['art'] = check_code(form, 'art', 'dokart', "Text ist: fehlt")
    dok['mtyp'] = cc('mimetyp','txt')
    dok['fname'] = '%s.txt' % dokid
    dok['zeit'] = int(time.time())
    dok.setDate('v',
                  check_date(form, 'v',
                             "Fehler im Datum"))
    text = check_str_not_empty(form, 'text', "Kein Text")
    akte_path = mk_akte_dir(akteold['id'])
    
    try:
        f = open('%s/%s' % (akte_path, dok['fname']), 'w')
        f.write(text)
        f.close()
        akte_path = get_akte_path(akteold['id'])
        os.chmod('%s/%s' % (akte_path, dok['fname']), 0600)
    except Exception, args:
        raise EBUpdateError("Fehler beim Anlegen der Datei: %s" % str(args))
    try:
        dok.insert(dokid)
    except Exception, args:
        try: dok.delete()
        except: pass
        raise EBUpdateError("Fehler beim Einfgen in die Datenbank: %s" % str(args))
        
    _stamp_akte(dok['akte'])
    
    
def updvermeinf(form):
    """Text ndern."""
    
    import os
    dokold = check_exists(form, 'dokid', Dokument, "Dokument-Id fehlt")
    dok = Dokument()
    dok['fall_id'] = check_fk(form, 'fallid', Fall, "Kein Fall", dokold)
    dok['mit_id'] = check_fk(form, 'mitid', Mitarbeiter,
                             "Kein Mitarbeiter", dokold)
    get_string_fields_new(dok, form, ['betr'], dokold, check_empty=True)
    dok['art'] = check_code(form, 'art', 'dokart', "Text ist: fehlt", dokold)
    dok['zeit'] = int(time.time())
    dok.setDate('v',
                check_date(form, 'v',
                           "Fehler im Datum"))
    if not is_binary(dokold):
        dok['mtyp'] = cc('mimetyp','txt')
        dok['fname'] = '%s.txt' % dokold['id']
        text = check_str_not_empty(form, 'text', "Kein Text")
        fall = Fall(dok['fall_id'])
        akteold = fall['akte']
        fname = os.path.join(get_akte_path(akteold['id']),
                             dok['fname'])
        try:
            f = open(fname, 'w+')
            f.write(text)
            f.close()
            os.chmod(fname,0600)
        except Exception, args:
            raise EBUpdateError("Fehler beim Anlegen der Datei: %s" % str(args))
    dokold.update(dok)
    _stamp_akte(dokold['akte'])
    
    
def removedoks(form):
    """Dokument lschen."""
    
    import os
    dokids = check_list(form, 'dokids', "Keinen Eintrag markiert?")
    for d in dokids:
        try:
            dok = Dokument(int(d))
            fall = Fall(dok['fall_id'])
            akteold = fall['akte']
            akte_path = get_akte_path(akteold['id'])
            os.remove('%s/%s' % (akte_path, dok['fname']))
            akte = Akte()
            akte['zeit'] = int(time.time())
            akteold.update(akte)
            dok.delete()
        except Exception, args:
            raise EBUpdateError("Fehler beim Loeschen, id: %s" % str(args))
            
            
def uploadeinf(form):
    """Upload eines Dokumentes."""
    
    import os
    
    if form.has_key('datei'):
        #print form.items()
        dokid = check_int_not_empty(form, 'dokid', "Dokumentenid fehlt")
        check_not_exists(dokid, Dokument,
                         "Dokument (id: %(id)s) existiert bereits")
        dok = Dokument()
        dok['fall_id'] = check_fk(form, 'fallid', Fall, "Kein Fall")
        fall = Fall(dok['fall_id'])
        akteold = fall['akte']
        dok['mit_id'] = check_fk(form, 'mitid', Mitarbeiter, "Kein Mitarbeiter")
        get_string_fields_new(dok, form, ['betr'], check_empty=True)
        dok['art'] = check_code(form, 'art', 'dokart', "Text ist: fehlt")
        dok.setDate('v',
                      check_date(form, 'v',
                               "Fehler im Datum"))
        dok['zeit'] = int(time.time())
        
        try:
            headers = form['datei'].headers
            if headers.has_key('Content-Type'):
                ctype = headers['Content-Type']
        except Exception, args:
            raise EE("Keine Dateiheaders erhalten: %s" % str(args))
            
        try:
            fname_orig = form['datei'].filename
        except Exception, args:
            raise EE("Kein Dateiname erhalten: %s" % str(args))
            
            #
            # Netscape sendet: Content-Type: application/xxx
            # Opera sendet: Content-Type: application/xxx; name="filename.ext"
            # Der Mime Typ wird hier nur von der .ext abgeleitetet !
            #
            
        try:
            i = string.rindex(fname_orig, '.')
            ext = fname_orig[i+1:]
            dok['mtyp'] = cc('mimetyp', '%s' % ext)
            ##       content_type = form['datei'].headers['Content-Type']
            ##       dok['mtyp'] = cn('mimetyp', content_type)
            dok['fname'] = '%s.%s' % (dokid, ext)
        except Exception, args:
            raise EE("Kein passendes Dateiformat (Mime Typ). %s" % str(args))
            
        try:
            f = open('%s/%s' % (mk_akte_dir(akteold['id']), dok['fname']), 'wb')
        except Exception, args:
            raise EBUpdateError("Fehler beim Oeffnen der Datei. %s" % str(args))
        try:
            f.write(form['datei'].read())
        except Exception, args:
            raise EBUpdateError("Fehler beim Speichern der Datei. %s" % str(args))
        try:
            f.close()
        except Exception, args:
            raise EBUpdateError("Fehler beim Schliessen der Datei. %s" % str(args))
        try:
            akte_path = get_akte_path(akteold['id'])
        except Exception, args:
            raise EBUpdateError("Fehler beim Finden des Dateipfades. %s" % str(args))
        try:
            os.chmod('%s/%s' % (akte_path, dok['fname']) ,0600)
        except Exception, args:
            raise EBUpdateError("Fehler beim Anlegen der Datei: %s" % str(args))
        try:
            dok.insert(dokid)
        except Exception, args:
            try: dok.delete()
            except: pass
            raise EBUpdateError("Fehler beim Einfgen in die Datenbank: %s" % str(args))
            
        _stamp_akte(dok['akte'])
        
        
def zdaeinf(form):
    """Neu z.d.A., Aktenabschluss."""
    
    fallold = check_exists(form, 'fallid', Fall, "Kein Fall")
    zustold = check_exists(form, 'aktuellzustid', Zustaendigkeit,
                        "Keine aktuelle Zustndigkeit")
    zdadatum =  check_date(form, 'zda',
                                "Fehler im zdA-Datum")
    fallende = fallold.getDate('zda')
    if fallende != Date(0,0,0):
        raise EE("Fall ist bereits zdA")
    zustende = zustold.getDate('e')
    if zustende != Date(0,0,0):
        raise EE("Zustndigkeit fr Fall ist bereits beendet")
    if zdadatum < zustold.getDate('bg'):
        raise EE("z.d.A.-Datum liegt vor Zustndigkeitsbeginn")
        
    fall = Fall()
    fall.setDate('zda', zdadatum)
    fall['status'] = cc('stand', 'zdA')
    zust = Zustaendigkeit()
    zust.setDate('e', zdadatum)
    fstatl = FachstatistikList(where = 'fall_id = %d' % fallold['id'])
    
    if len(fstatl) == 1:
        fs = fstatl[0]
        if fs['jahr'] != zdadatum.year:
            raise EE("Jahr der Fachstatistik entspricht nicht dem Jahr des Fallabschlusses")
    elif len(fstatl) > 1:
        raise EE("Mehr als 1 Fachstatistik fr Fallnummer '%s', " % fallold['fn']
                 + "'%d' vorhanden." % fall['zday'] )
    else:
        raise EE("Keine Fachstatistik fr Fallnummer '%s', " % fallold['fn'])
        
##     jghstatl = JugendhilfestatistikList(where = 'fall_id = % d' % fallold['id'] )
##     if len(jghstatl) == 1:
##         pass
##     elif len(jghstatl) > 1:
##         raise EE("Mehr als eine Jugendhilfestatistik fr Fallnummer '%(fn)s' vorhanden." % fallold )
##     else:
##         raise EE("Keine Jugendhilfestatistik fr Fallnummer '%(fn)s' vorhanden" % fallold )
    jgh = fallold['jgh']
    if jgh and isinstance(jgh, Jugendhilfestatistik2007):
        if jgh['hda'] == cc('ja_nein', '1'):
            raise EE("Die vorhandene Jugendhilfestatistik ist nicht fr einen abgeschlossenen Fall.")
        if jgh['jahr'] != zdadatum.year:
            raise EE("Jahr der Bundesstatistik entspricht nicht dem Jahr des Fallabschlusses")


    if hasattr(config, 'KEINE_BUNDESSTATISTIK') and config.KEINE_BUNDESSTATISTIK:
        pass
    elif not jgh:
        raise EE("Keine Jugendhilfestatistik fr Fallnummer '%(fn)s' vorhanden" % fallold )

    if config.WIREB:
        try:
            wireb = fallold['wireb'][0]
            if not wireb['alle_bearbeitet__code'] == '1':
                raise EE("Wir.EB: Evaluationsbgen noch nicht ausgefllt fr Fallcodenummer {}. "
                         "Bitte ausfllen und Hkchen setzten.".format(wireb['fallcodenummer']))
            jahr = wireb['jghbgy']
            if jahr is None or jahr == 0:
                raise EE("Wir.EB: Bundesstatistik noch nicht hochgeladen fr Fallcodenummer {}. "
                         "Bitte hochladen.".format(wireb['fallcodenummer']))
        except IndexError:
            # keine Wir.EB-Anbindung
            pass
    akteold = fallold['akte']
    letzter_fall = akteold['letzter_fall']
    beratungen = BeratungskontaktList(
        where='fall_id = {}'.format(fallold['id']),
        join=[('fallberatungskontakt',
               'beratungskontakt.id = fallberatungskontakt.bkont_id')])
    for b in beratungen:
        if b.getDate('k') > zdadatum:
            raise EE("Fallabschlussdatum liegt vor mindestens einem Beratungskontakt.")
    leistungen = LeistungList(where = 'fall_id = %s' % letzter_fall['id'])
    for s in leistungen:
        beginn = s.getDate('bg')
        ende = s.getDate('e')
        if beginn > zdadatum:
            raise EE("Fallabschlussdatum liegt vor dem Beginn mindestens einer Leistung.")
        if ende.year != 0 and ende > zdadatum:
            raise EE("Fallabschlussdatum liegt vor dem Ende mindestens einer Leistung.")
    leistungen = LeistungList(where = 'fall_id = %s and ey = 0' % letzter_fall['id'])
    for s in leistungen:
        leist = Leistung()
        leist.setDate('e', zdadatum)
        s.update(leist)
    zustold.update(zust)
    fallold.update(fall)
    ZusaetzlicheBerechtigungList(
        where='fall_id = {}'.format(fallold['id'])).deleteall()
    # akte.sperre auf null stellen, damit wird die Akte nach 
    # Ablauf der SPERRFRIST (falls definiert) wieder automatisch gesperrt
    akteold.update({'sperre': None})
    _stamp_akte(fallold['akte'])
    


def _check_fallbeginn(fall, fallbeginn):
    faelle = fall['akte__faelle'].sorted('bgy', 'bgm', 'bgd')
    i = faelle.index(fall)
    if i > 0:
        # nicht der erste Fall
        if fallbeginn < faelle[i-1].getDate('zda'):
            raise EE('Fallbeginn vor Ende des vorherigen Falls')
    zustaendigkeiten = fall['zustaendigkeiten'].sorted('bgy', 'bgm', 'bgd')
    if fallbeginn > zustaendigkeiten[0].getDate('e'):
            raise EE('Fallbeginn nach Ende der ersten Zustndigkeit')
    for leist in fall['leistungen']:
        if leist.getDate('bg') < fallbeginn:
            raise EE('Fallbeginn nach Beginn der ersten Leistung')
    if int(fall['fn'].split('-')[1][0:4]) != fallbeginn.year:
        raise EE('Das Jahr des Fallbeginns muss mit dem Jahr in der Fallnummer bereinstimmen')
            
    
def updfall(form):
    """Update Fall."""
    fallold = check_exists(form, 'fallid', Fall, "Keine Fallid")
    erste_zustold = fallold['zustaendigkeiten'].sorted('bgy', 'bgm', 'bgd')[0]
    fall = Fall()
    zust = Zustaendigkeit()
    fallbeginn = check_date(form, 'bg', "Fehler im Fallbeginndatum", fallold)
    _check_fallbeginn(fallold, fallbeginn)
    fall.setDate('bg', fallbeginn)
    zust.setDate('bg', fallbeginn)
    # Das soll wohl hier nicht gendert werden
    # fall['status'] = 236
    
    fallold.update(fall)
    erste_zustold.update(zust)
    _stamp_akte(fallold['akte'])
    
    
def zdareinf(form):
    """z.d.A. rckgngig machen."""
    
    # es werden nur das Fallendedatum und das Zustndigkeitsende wieder auf offen
    # gesetzt. Keine neue Zustndigkeit.
    fallold = check_exists(form, 'fallid', Fall, "Keine Fallid")
    zustold = fallold['zuletzt_zustaendig']
    if fallold['aktuell']:
        raise EE('Kein Zda-Rckgngig fr aktuellen Fall')
    zda_datum = fallold.getDate('zda')
    leistungenold = [l for l in fallold['leistungen'] if l.getDate('e') == zda_datum]
    fall = Fall()
    zust = Zustaendigkeit()
    fall.setDate('zda', Date(0,0,0))
    zust.setDate('e', Date(0,0,0))
    # alle Leistungen, die am alten zda-Datum beendet wurden, wieder ffnen
    for l in leistungenold:
        leist = Leistung()
        leist.setDate('e', Date(0,0,0))
        l.update(leist)
    fall['status'] = cc('stand', 'l')
    fallold.update(fall)
    zustold.update(zust)
    _stamp_akte(fallold['akte'])
    
    
def waufneinf(form):
    """Wiederaufnahme der Akte."""
    # die Wiederaufnahmefrist wird hier berhaupt nicht geprft, das
    # sollte aber der Fall sein.
    # Bis jetzt wird alles ber den Aufbau der Klientenkarte entschieden
    # (zda rckgngig Button oder Wiederaufnahmenbutten)
    akteold = check_exists(form, 'akid', Akte, "Aktenid fehlt")
    stelle = Code(akteold['stzbg'])
    akte = Akte()
    get_string_fields_new(akte, form, ['na'], akteold, check_empty=True)
    akte['gs'] = check_code(form,'gs', 'gs',
                            "Keine Geschlechtsangabe")
    akte['gb'] = str(check_date(form, 'gb', 'Fehler im Geburtsdatum'))
    get_string_fields_new(akte, form,
                          ['vn', 'ber', 'tl1', 'tl2', 'mail', 'no'], akteold)
    akte['aufbew'] = check_code(form,'aufbew', 'aufbew',
                            "Keine Aufbewahrungskategorie")
    get_int_fields(akte, form, ['abwlf'])
    abwlf = akte['abwlf']
    if abwlf is not None and abwlf < config.MINIMALE_LOESCHFRIST:
        raise EE("Minimale abweichende Lschfrist ist {} Monate.".format(config.MINIMALE_LOESCHFRIST))
    akte['fs'] = check_code(form,'fs', 'fsfs',
                            "Fehler im Familienstatus", akteold)
    setAdresse(akte, form)
    fallid = check_int_not_empty(form, 'fallid', "Fallid fehlt")
    
    letzter_fall = akteold['letzter_fall']
    fall = Fall()
    fall['akte_id'] = check_fk(form, 'akid', Akte,
                              "Keine Aktenid fr Fall")
    # Fallbeginn ist identisch mit Zustndigkeitsbeginn
    fall.setDate('bg',
                 check_date(form, 'zubg',
                            "Fehler im Datum fr den Zustndigkeitsbeginn"))
    if fall.getDate('bg') <= letzter_fall.getDate('zda'):
        raise EE('Fallbeginn muss nach Abschluss des letzten Falls liegen')
    if config.ANMELDEPROBLEME_UND_BESCHAEFTIGUNG_BEI_ANMELDUNG_ERFASSEN:
        # qualij auf Schule setzen, falls nicht explizit angegeben und Alter <= 14 Jahre
        if form.get('qualij') == ' ':
            alter = calc_age(akte['gb'], fall.getDate('bg'))
            if alter < 14:
                form['qualij'] = cc('fsqualij', '7') # entfllt, da unter 14
        for feld, kat_code, name in (('qualij', 'fsqualij', 'Beschftigung des/r Jugendlicher/n'),
                                     ('bkm', 'fsbe', 'Beschftigungsverhltnis der Mutter'),
                                     ('bkv', 'fsbe', 'Beschftigungsverhltnis des Vaters'),):
            #print 'FORM', form
            #print 'FELD KAT_CODE NAME', feld, kat_code, name
            akte[feld] = check_code(form, feld,
                                    kat_code,
                                    "Fehlt: %s" % name)
    fall.setDate('zda', Date(0,0,0))
    fall['fn'] = getNewFallnummer(stelle['code'], fall['bgy'])
    fall['status'] = cc('stand', 'l')
    
    zust = Zustaendigkeit()
    zust['mit_id'] = check_fk(form, 'zumitid', Mitarbeiter,
                              "Kein zustndiger Mitarbeiter fr Fall")
    zust.setDate('bg', fall.getDate('bg'))
    zust.setDate('e', Date(0,0,0))
    
    leist = Leistung()
    leist['mit_id'] = check_fk(form, 'lemitid', Mitarbeiter,
                              "Kein zustndiger Mitarbeiter fr Leistung")
    leist['le'] = check_code(form, 'le', 'fsle', "Keine Leistungsart")
    leist.setDate('bg',
                  check_date(form, 'lebg',
                             "Fehler im Datum fr den Leistungsbeginn"))
    if leist.getDate('bg') < fall.getDate('bg'):
        raise EE("Datum des Leistungs- vor Zustaendigkeitsbeginn")
    leist.setDate('e', Date(0,0,0))
    leist['stz'] = check_code(form, 'lestz', 'stzei',
                              "Kein Stellenzeichen fr die Leistung",
                              stelle['id'])
    if config.ANMELDUNGSDATEN_OBLIGATORISCH:
        anm = _check_anmeinf(form)
    try:
        akteold.update(akte)
        fall.insert(fallid)
        if config.ANMELDUNGSDATEN_OBLIGATORISCH:
            anm['fall_id'] = fall['id']
            anm.insert()
        zust['fall_id'] = fall['id']
        zust.new()
        zust.insert()
        leist['fall_id'] = fall['id']
        leist.new()
        leist.insert()
    except Exception, args:
        try: fall.delete()
        except: pass
        try: zust.delete()
        except: pass
        try: leist.delete()
        except: pass
        raise EBUpdateError("Fehler beim Einfgen in die Datenbank: %s" % str(args))
    _stamp_akte(akteold)
    

def _fs_check(form, fstat, fstatold=None):
    fstat['mit_id'] = check_fk(form, 'mitid', Mitarbeiter, "Kein Mitarbeiter")
    jahr = fstat['jahr'] = check_int_not_empty(form, 'jahr', "Jahr fehlt")
    fstat['stz'] = check_code(form, 'stz', 'stzei',
                              "Kein Stellenzeichen fr die Fachstatistik")
    # Regionalinfos bernehmen
    fall_id = fstat['fall_id']
    fall = None
    fall_aktuell = False
    if fall_id:
        fall = Fall(fall_id)
        akte = fall['akte']
        fall_aktuell = fall['aktuell']
        # Regionaldaten bernehmen:
        # - bei laufenden Fllen immer
        # - bei abgeschlossenen Fllen nur wenn vorher gar kein Ort bekannt war
        # Wird in ebkus.html.fachstatistik.regional_daten() implementiert.
        from ebkus.html.fachstatistik import regional_daten
        rd = regional_daten(fstatold, akte, fall)
        for k, v in rd.items():
            fstat[k] = v
    if not fall_aktuell:
        # Warnen beim Speichern einer Fachstatistik fr einen abgeschlossene Fall
        # Geht nur, wenn vorher Hkchen bei 'Vernderung der Fachstatistik fr einen abgeschlossenen Fall zulassen'
        # gemacht wurde.
        modi_zulassen = form.get('modi_zulassen')
        if modi_zulassen != '1':
            warning = "Sie verndern die Fachstatistik eine abgeschlossenen Falles.<br /><br /><br />"
            warning += "Wenn Sie sicher sind, dass Sie das tun wollen, setzen Sie ein Hkchen bei " 
            warning += "<em>Vernderung der Fachstatistik fr einen abgeschlossenen Fall zulassen</em>.<br /><br /><br />"
            raise EE(warning)
    if fall_aktuell:
        if jahr > today().year:
            raise EE('Fehler im Jahr: zu weit in der Zukunft')
        if jahr < fall['bgy']:
            raise EE("Fehler im Jahr: liegt vor Fallbeginn")
    elif fall:
        if jahr != fall['zday']:
            raise EE("Fehler im Jahr: Jahr fr abgeschlossene Flle muss gleich dem Jahr des Fallabschlusses sein")
        
    # ab hier abschaltbar
    from ebkus.html.fskonfig import fs_customize as fsc
    multicode = cc('verwtyp', 'm')
    singlecode = cc('verwtyp', 'k')
    for fname in fsc.abschaltbare_items:
        if not fsc.deaktiviert(fname):
            feld = fsc.get(fname) # das Feldobjekt
            verwtyp = feld['verwtyp']
            if verwtyp == singlecode:
                fstat[fname] = check_code(form, fname,
                                          feld['kat_code'],
                                          "Fehlt: %s" % feld['name'])
            elif verwtyp == multicode:
                fstat[fname] = check_multi_code(form, fname,
                                                feld['kat_code'],
                                                "Fehlt: %s" % feld['name'])
            elif fname in ('no', 'no2', 'no3'):
                get_string_fields_new(fstat, form, [fname], '')
            elif fname in ('kat',):
                uebernehmen = form.get('uebernehmen')
                if uebernehmen == '1' and config.BERATUNGSKONTAKTE and not config.BERATUNGSKONTAKTE_BS:
                    # aus Beratungskontakten bernehmen
                    from ebkus.html.beratungskontakt import get_fs_kontakte
                    get_fs_kontakte(fall, fstat)
                else:
                    fstat[fname] = check_int_not_empty(form, fname, "Terminsumme fehlt")
                    for f in fsc.termin_felder:
                        fobj = fsc.get(f)
                        fstat[f] = check_int_not_empty(form, f, "Fehlt: %s" % fobj['name'])
##             elif fname in fsc.termin_felder:
##                 fstat[fname] = check_int_not_empty(form, fname, "Keine Terminanzahl", 0)

                        
    if fall_aktuell and config.WARNUNG_BEI_FACHSTATISTIK_AKTE_DISKREPANZ:
        # abgeschlossene Flle knnen diskrepant sein
        # geprft wird: eleistungen, fs, zm, anmprobleme, qualij, bkm, bkv
        diskr = form.get('diskr')
        warnen = False
        if diskr != '1' and fall:
            warning = "Es besteht eine Diskrepanz zwischen Angaben in der Fachstatistik " + \
                      "und in der Klientenakte.<br /><br /><br />"
            warning += "<table>"
            warning += '<tr><th>Angaben in der Akte:</th><th></th></tr><tr><td>&nbsp;</td><td>&nbsp;</td></tr>'
            try:
                a = set()
            except:
                from sets import Set as set
            fall_leistungen = set([str(leist['le']) for leist in fall['leistungen']])
            if not fsc.deaktiviert('eleistungen'):
                fs_leistungen = set(fstat.get('eleistungen', '').split())
                if fall_leistungen != fs_leistungen:
                    leistungen_name = list(set([leist['le__name'] for leist in fall['leistungen']]))
                    leistungen_name.sort()
                    warnen = True
                    warning += "<tr><td><strong>Erbrachte Leistungen:</strong></td>"
                    if leistungen_name:
                        warning += "<td>%s</td></tr>" % leistungen_name[0]
                        for leist in leistungen_name[1:]:
                            warning += "<tr><td></td><td>%s</td></tr>" % leist
            if not fsc.deaktiviert('fs'):
                if fall['akte__fs'] != fstat.get('fs'):
                    warnen = True
                    warning += "<tr><td><strong>Familienstatus:</strong> </td><td>%s</td></tr>" % \
                        fall['akte__fs__name']
            if not fsc.deaktiviert('gs'):
                if fall['akte__gs'] != fstat.get('gs'):
                    warnen = True
                    warning += "<tr><td><strong>Geschlecht:</strong> </td><td>%s</td></tr>" % \
                        fall['akte__gs__name']
            anmeldungen = fall['anmeldung']
            if anmeldungen:
                anm = anmeldungen[0]
                if not fsc.deaktiviert('zm'):
                    if anm['zm'] != fstat.get('zm'):
                        warnen = True
                        warning += "<tr><td><strong>Empfohlen von:</strong></td><td>%s</td></tr>" % \
                            anm['zm__name']
                if config.ANMELDEPROBLEME_UND_BESCHAEFTIGUNG_BEI_ANMELDUNG_ERFASSEN:
                    for feld, kat_code, name in (('qualij', 'fsqualij', 'Beschftigung des/r Jugendlicher/n'),
                                                 ('bkm', 'fsbe', 'Beschftigungsverhltnis der Mutter'),
                                                 ('bkv', 'fsbe', 'Beschftigungsverhltnis des Vaters'),):
                        if not fsc.deaktiviert(feld):
                            if akte[feld] != fstat.get(feld):
                                warnen = True
                                warning += "<tr><td><strong>%s:</strong></td><td>%s</td></tr>" % \
                                    (name, akte['%s__name' % feld] or '')

                    if not fsc.deaktiviert('anmprobleme'):
                        if anm['anmprobleme'] != fstat.get('anmprobleme'):
                            warnen = True
                            _anmprobleme = anm['anmprobleme'] or ''
                            warning += "<tr><td><strong>Problem(e) bei der Anmeldung:</strong></td><td>%s</td></tr>" % \
                                ('<br/>'.join([Code(int(i))['name'] for i in _anmprobleme.split()]),)
            warning += "</table><br /><br /><br />"
            warning += "Bitte berprfen Sie, welches die richtigen Angaben sind.<br /><br /><br />"
            warning += "Wenn sie wirklich diese Diskrepanz abspeichern mchten, markieren Sie \"Diskrepanz zur Akte zulassen\".<br /><br /><br />"

            if warnen:
                raise EE(warning)
    if fall_id and akte:
        akte.akte_undo_cached_fields()

        

def fseinf(form):
    """Neue Fachstatistik."""
    
    fsid = check_int_not_empty(form, 'fsid', "FachstatistikID fehlt")
    
    check_not_exists(fsid, Fachstatistik,
      "Fachstatistik (id: %(id)s, Fallnummer: %(fall_fn)s, Mitarbeiter: %(mit_id__na)s) existiert bereits")
    fstat = Fachstatistik()
    if form['fallid'] != None and form['fallid'] != '':
        fstat['fall_id'] = check_fk(form, 'fallid', Fall, "Kein Fall")
    fstat['fall_fn'] = check_str_not_empty(form, 'fall_fn', "Fallnummer fehlt")
    jahresl = FachstatistikList(where = "fall_fn = '%s'" % fstat['fall_fn'])
    if len(jahresl) >= 1:
        raise EE("Fachstatistik fr Fallnummer: '%s' vorhanden" % fstat['fall_fn'])
    _fs_check(form, fstat)
    fstat['zeit'] = int(time.time())
    try:
        fstat.insert(fsid)
    except Exception, args:
        try: fstat.delete()
        except: pass
        raise EBUpdateError("Fehler beim Einfgen in die Datenbank: %s" % str(args))
        
        
def updfs(form):
    """Update der Fachstatistik."""
    
    fstatold = check_exists(form, 'fsid', Fachstatistik, "Fachstatistikid fehlt")
    fstat = Fachstatistik()
    fallid = form.get('fallid')
    if fallid == None or fallid == '' or fallid == 'None':
        fstat['fall_id'] = None
    else:
        fstat['fall_id'] = check_fk(form, 'fallid', Fall, "Kein Fall")
    fstat['fall_fn'] = check_str_not_empty(form, 'fall_fn', "Fallnummer fehlt")
    #fstat['bz'] kann nicht updatet werden
    _fs_check(form, fstat, fstatold=fstatold)
    fstat['zeit'] = int(time.time())
    fstatold.update(fstat)
    ## vgl. Kommentar bei updjgh
    ##
    if fstatold['fall_id'] != None:
        _stamp_akte(fstatold['akte'])
        

def updfskonfig(form):
    """Fachstatistikformular konfigurieren.
    Das Formular schickt grundstzlich immer Daten fr alle Felder!
    """
    def set_aktiviert(f_obj, status):
        flag = f_obj['flag']
        if status:
            # aktivieren
            f_obj.update({'flag': flag&~1})
        else:
            # deaktivieren
            f_obj.update({'flag': flag|1})
    from ebkus.html.fskonfig import fs_customize as fsc
    for f in fsc.abschaltbare_items:
        f_obj = fsc.get(f)
        status = form.get('%s_a' % f)
        set_aktiviert(f_obj, status)
        if f == 'kat':
            # alle Terminfelder richten sich nach kat, der Summe
            for t in fsc.termin_felder:
                t_obj = fsc.get(t)
                set_aktiviert(t_obj, status)
        if f in fsc.joker_felder:
            label = form.get('%s_l' % f)
            f_obj.update({'name': label})
            kat_id = form.get('%s_k' % f)
            kat = Kategorie(kat_id)
            f_obj.update({'kat_code': kat['code']})
            f_obj.update({'kat_id': kat['id']})

# Mehrfach-Joker zur Zeit nicht implementiert, da die Integer-Felder fr Joker
# in der Tabelle fachstat das nicht hergeben. Eine Lsung wre, zustzliche
# Joker-Felder in fachstat anzulegen, mit VARCHAR/verwtyp=m
#         if f in fsc.joker_frei:
#             mehrfach = form.get('%s_m' % f)
#             if mehrfach:
#                 f_obj.update({'verwtyp': cc('verwtyp', 'm')})
#             else:
#                 f_obj.update({'verwtyp': cc('verwtyp', 'k')})
        if f in fsc.joker_frei:
            mehrfach = form.get('%s_m' % f)
            flag = f_obj['flag']
            # Prsenz von bit 2 signalisiert einfach
            if mehrfach:
                #print 'MERHFACH'
                f_obj.update({'flag': flag&~2})
            else:
                #print 'NICHT MERHFACH'
                f_obj.update({'flag': flag|2})

    

def raise_if_int(form, keys, error_message):
    for k in keys:
        v = form.get(k)
        found = False
        try:
            vi = int(v)
            found = True
        except (ValueError, TypeError):
            pass
        if found:
            raise EE(error_message)

def _jgh07_check(form, jgh):
    """Jugendhilfestatistik prfen sowohl fr Einfhrung als auch fr Update."""
    v1 = form.get('v1')
    if v1:
        # zum Testen mit speziellem Parameter v1
        _jgh07_check_v1(form, jgh)
    else:
        # Standard mit jghdef
        _jgh07_check_v2(form, jgh)
    # from ebkus.utils import write_dict_to_file
    # write_dict_to_file(jgh, dir=config.INSTANCE_HOME, prompt="jgh_{}".format('v2' if v2 else 'v1'))
    

def _jgh07_check_v1(form, jgh):
    """
    Jugendhilfestatistik prfen sowohl fr Einfhrung als auch fr Update
    nach der alten Methode. Nur noch fr Testzwecke
    """
    v2 = form.get('v2')
    if v2:
        return _jgh07_check_v2(form, jgh)
    from ebkus.html.jghstatistik import _fb_data
    try:
        fall_id = check_fk(form, 'fallid', Fall, "Kein Fall")
        fall = Fall(fall_id)
        fall_aktuell = fall['aktuell']
    except:
        # es muss keinen Fall geben, z.B. falls dieser schon gelscht wurde
        fall = None
        fall_aktuell = False
    if not fall_aktuell:
        # Warnen beim Speichern einer Jugendhilfestatistik fr einen abgeschlossene Fall
        # Geht nur, wenn vorher Hkchen bei 'Vernderung der Jugendhilfestatistik fr einen abgeschlossenen Fall zulassen'
        # gemacht wurde.
        modi_zulassen = form.get('modi_zulassen')
        if modi_zulassen != '1':
            warning = "Sie verndern die Jugendhilfestatistik eines abgeschlossenen Falles.<br /><br /><br />"
            warning += "Wenn Sie sicher sind, dass Sie das tun wollen, setzen Sie ein Hkchen bei " 
            warning += "<em>Vernderung der Jugendhilfestatistik fr einen abgeschlossenen Fall zulassen</em>.<br /><br /><br />"
            raise EE(warning)
    jgh['jahr'] = jahr = check_int_not_empty(form, 'jahr', 'Kein Jahr')
    if jahr > today().year:
        raise EE('Fehler im Jahr: zu weit in der Zukunft')
    jgh['stz'] = check_code(form, 'stz', 'stzei',
                            "Kein Stellenzeichen fr die Jugendhilfestatistik")
    jgh['mit_id'] = check_fk(form, 'mitid', Mitarbeiter,
                                 "Kein Mitarbeiter")
    jgh['land'] = check_code(form, 'land', 'land',
                            "Kein Land")
    jgh['kr'] = check_code(form, 'kr', 'kr',
                            "Kein Kreis")
    jgh['einrnr'] = check_code(form, 'einrnr', 'einrnr',
                            "Keine Einrichtungsnummer")
    jgh['gfall'] = check_code(form, 'gfall', 'gfall',
                                  "Angabe, ob der Fall ein Geschwisterfall ist, fehlt")
    # wird auf True gesetzt, wenn das entsprechende item mit ja beantwortet wurde    
    hilfe_dauert_an = False 
    # Kategorienitems werden ber die Daten in jghstatistik._fb_data abgehandelt
    for a in _fb_data:
        if isinstance(a, basestring):
            continue
        lfb = a['abschnitt']
        title = a['title']
        if lfb == 'K':
            _jgh07_check_abschnitt_K(form, jgh)
        else:
            for i in a['items_data']:
                if isinstance(i, basestring):
                    continue
                typ = i['typ'][0]
                name = i['name']
                frage = i['frage']
                if typ == 'kat':
                    if name == 'gef8a' and jahr < 2012: # erst ab 2012 verbindlich eingefhrt
                        continue
                    if name == 'inobh42_22':
                        if  jahr < 2017: # erst ab 2017 verbindlich eingefhrt
                            continue
                    kat_code = i['typ'][1]
                    item_kennzeichung = "%s / %s / %s" % (lfb, title, frage)
                    if name == 'hda' and str(form.get('hda')) == str(cn('ja_nein', 'ja')):
                        hilfe_dauert_an = True
                    if hilfe_dauert_an and name in ('lbk6m', 'grende', 'aort_nac', 'unh'):
                        raise_if_int(form, (name,),
                                     "Nur bei beendeter Hilfe ausfllen: %s" %
                                     item_kennzeichung)
                        jgh[name] = None
                    else:
                        value = check_code(form, name, kat_code,
                                           "Fehlt: %s" % item_kennzeichung)
                        if name == 'inobh42_22' and jahr < 2022:
                            if value in (cc('inobh42', '2'), cc('inobh42', '3')):
                                raise EE("Fr<br/><br/>%s<br/><br/>stand<br/><br/>%s<br/><br/>als Antwortmglichkeit vor 2022 nicht zur Verfgung" % (item_kennzeichung, Code(value)['name']))
                        jgh[name] = value

    jgh['zustw'] = form.get('zustw') or ' '
    # Falls "Gefhrdungseinschtzung nach  8a" mit ja beantwortet wurde,
    # muss als als "anregende Person
    # oder Institution" ein "Soziale/r Dienst/e und andere Institution/en
    # (z.B. Jugendamt)" angegeben sein.
    # import logging
    # logging.info("gef8a: %s cc('ja_nein', '1'): %s aip: %s cc('aip', '4'): %s" % 
    #              (jgh.get('gef8a'), cc('ja_nein', '1'), jgh['aip'], cc('aip', '4')))
    if jahr >= 2012 and jgh.get('gef8a') == cc('ja_nein', '1'):
        if not jgh['aip'] == cc('aip', '4'):
            raise EE("Bei Einleitung der Hilfe auf Grund einer vorangegangenen Gefhrdungseinschtzung gem  8a Absatz 1 SGB VIII "
                     'muss als anregende Institution "Soziale/r Dienst/e und andere Institution/en (z.B. Jugendamt)" angegeben werden.')

    # Datumsangaben direkt abhandeln
    jgh.setDate('ge',
                check_date(form, 'ge',
                           "Fehler in E / Geburtsdatum", nodayallowed = 1 ) )
    del jgh['ged']
    jgh.setDate('bg',
                check_date(form, 'bg',
                           "Fehler im Datum fr den Beginn", nodayallowed = 1 ) )
    if fall and (fall['bgy'] > jgh['bgy'] or
                 (fall['bgy'] == jgh['bgy'] and fall['bgm'] > jgh['bgm'])):
        raise EE("Beginndatum in der Jugendhilfestatistik vor Fallbeginn")
    if jahr < jgh['bgy']:
        raise EE("Fehler im Jahr: liegt vor Fallbeginn")
    if hilfe_dauert_an:
        raise_if_int(form, ('ey', 'em'),
                     "Nur bei beendeter Hilfe ausfllen: L / Ende der Hilfe / Datum")
        jgh['ey'] = jgh['em'] = None
    else:
        jgh.setDate('e',
                    check_date(form, 'e',
                               "Fehler in L / Ende der Hilfe / Datum", nodayallowed = 1 ) )
        if jgh.getDate('e') < jgh.getDate('bg'):
            raise EE("Fallende in der Jugendhilfestatistik liegt vor Fallbeginn")
        if fall and jgh.getDate('e') > fall.getDate('zda'):
            raise EE("Ende der Beratung darf nicht nach dem Abschluss des Falles liegen.")
        if jgh.getDate('e') < Date(2007):
            raise EE("Neue Bundesstatistik fr Fallende vor 2007 nicht erlaubt")
        if jgh.getDate('e').year != jahr:
            raise EE("Jahr des Endes der Beratung nicht gleich dem Jahr")
        # Soll verhindern, dass man ey und jahr gemeinsam nach dem Fallabschluss verschiebt.
        if fall and fall['zday'] > 0 and jahr > fall['zday']:
            raise EE("Fehler im Jahr: Jahr fr abgeschlossene Flle darf nicht nach dem Jahr des Fallabschlusses sein")
        del jgh['ed']
    del jgh['bgd']
    # Beratungskontakte direkt abhandeln
    item_J = "J / Intensitt / Zahl der Beratungskontakte"
    item_M = "M / Betreuungsintensitt / Zahl der Beratungskontakte"
    nbkakt = nbkges = None
    if config.BERATUNGSKONTAKTE:
        from ebkus.html.beratungskontakt import get_jgh_kontakte
        nbkakt, nbkges = get_jgh_kontakte(fall, jahr)
    #print 'DEFAULTS', nbkakt, nbkges
    if hilfe_dauert_an:
        jgh['nbkakt'] = check_int_not_empty(form, 'nbkakt',
                            "Fehlt: %s" % item_J, default=nbkakt)
        raise_if_int(form, ('nbkges',),
                     "Nur bei beendeter Hilfe ausfllen: %s" % item_M)
        jgh['nbkges'] = None
        assert isinstance(jgh['nbkakt'], (int, long))
        if jgh['nbkakt'] < 1:
            raise EE("Es muss mindestens 1 Beratungskontakt geben: %s" % item_J)
    else:
        jgh['nbkges'] = check_int_not_empty(form, 'nbkges',
                            "Fehlt: %s" % item_M, default=nbkges)
        raise_if_int(form, ('nbkakt',),
                     "Nur bei andauernder Hilfe ausfllen: %s" % item_J)
        jgh['nbkakt'] = None
        assert isinstance(jgh['nbkges'], (int, long))
        if jgh['nbkges'] < 1:
            raise EE("Es muss mindestens 1 Beratungskontakt geben: %s" % item_M)
    if fall:
        fall['akte'].akte_undo_cached_fields()

def _jgh07_check_v2(form, jgh):
    """
    Jugendhilfestatistik prfen sowohl fr Einfhrung als auch fr Update
    unter Verwendung von jghdef.
    """
    jgh['jahr'] = jahr = check_int_not_empty(form, 'jahr', 'Kein Jahr')
    if jahr < 2022:
        raise EE('Neueinfhrung bzw. Update der Jugendhilfestatistik fr Berichtsjahre vor 2022 nicht mglich')
    jghdef = get_jghdef(jahr, strict=False, raise_exception=True)
    try:
        fall_id = check_fk(form, 'fallid', Fall, "Kein Fall")
        fall = Fall(fall_id)
        fall_aktuell = fall['aktuell']
    except:
        # es muss keinen Fall geben, z.B. falls dieser schon gelscht wurde
        fall = None
        fall_aktuell = False
    if not fall_aktuell:
        # Warnen beim Speichern einer Jugendhilfestatistik fr einen abgeschlossene Fall
        # Geht nur, wenn vorher Hkchen bei 'Vernderung der Jugendhilfestatistik fr einen abgeschlossenen Fall zulassen'
        # gemacht wurde.
        modi_zulassen = form.get('modi_zulassen')
        if modi_zulassen != '1':
            warning = "Sie verndern die Jugendhilfestatistik eines abgeschlossenen Falles.<br /><br /><br />"
            warning += "Wenn Sie sicher sind, dass Sie das tun wollen, setzen Sie ein Hkchen bei " 
            warning += "<em>Vernderung der Jugendhilfestatistik fr einen abgeschlossenen Fall zulassen</em>.<br /><br /><br />"
            raise EE(warning)
    if jahr > today().year:
        raise EE('Fehler im Jahr: zu weit in der Zukunft')
    jgh['stz'] = check_code(form, 'stz', 'stzei',
                            "Kein Stellenzeichen fr die Jugendhilfestatistik")
    jgh['mit_id'] = check_fk(form, 'mitid', Mitarbeiter,
                                 "Kein Mitarbeiter")
    jgh['gfall'] = check_code(form, 'gfall', 'gfall',
                                  "Angabe, ob der Fall ein Geschwisterfall ist, fehlt")
    for item in jghdef.fb_item_iter():
        value = item.get_and_check_value(form)
        # TODO
        # print item.ebkus_name, value
        jgh[item.ebkus_name] = value
    gruende_item = jghdef.get_item(ebkus_name='gr1')
    v1, v2, v3 = jgh.get('gr1'), jgh.get('gr2'), jgh.get('gr3')
    if (v1 and (v1 == v2 or v1 == v3)) or (v2 and v2 == v3):
        raise EE(gruende_item.item_message("Mehrfach derselbe Grund"))
    if v3 and not v2:
        raise EE(gruende_item.item_message("Kein 2. Grund obwohl ein 3. Grund angegeben wurde"))
    # Falls "Gefhrdungseinschtzung nach  8a" mit ja beantwortet wurde,
    # muss als als "anregende Person
    # oder Institution" ein "Soziale/r Dienst/e und andere Institution/en
    # (z.B. Jugendamt)" angegeben sein.
    # import logging
    # logging.info("gef8a: %s cc('ja_nein', '1'): %s aip: %s cc('aip', '4'): %s" % 
    #              (jgh.get('gef8a'), cc('ja_nein', '1'), jgh['aip'], cc('aip', '4')))
    if jahr >= 2012 and jgh.get('gef8a') == cc('ja_nein', '1'):
        if not jgh['aip'] == cc('aip', '4'):
            raise EE("Bei Einleitung der Hilfe auf Grund einer vorangegangenen Gefhrdungseinschtzung gem  8a Absatz 1 SGB VIII "
                     'muss als anregende Institution "Soziale/r Dienst/e und andere Institution/en (z.B. Jugendamt)" angegeben werden.')
    # Die Ziffern sind schon gesetzt und der Wertebereich ist berprft.
    # print 'FALLBEGINN:', fall.getDate('bg')
    # print jgh
    # print 'JGHBEGINN:', jgh.getDate('bg')
    if fall and (fall['bgy'] > jgh['bgy'] or
                 (fall['bgy'] == jgh['bgy'] and fall['bgm'] > jgh['bgm'])):
        raise EE("Beginndatum in der Jugendhilfestatistik vor Fallbeginn")
    if jahr < jgh['bgy']:
        raise EE("Fehler im Jahr: liegt vor Fallbeginn")
    if jgh.get('ey'):
        if jgh.getDate('e') < jgh.getDate('bg'):
            raise EE("Fallende in der Jugendhilfestatistik liegt vor Fallbeginn")
        if fall and jgh.getDate('e') > fall.getDate('zda'):
            raise EE("Ende der Beratung darf nicht nach dem Abschluss des Falles liegen.")
        if jgh.getDate('e') < Date(2007):
            raise EE("Neue Bundesstatistik fr Fallende vor 2007 nicht erlaubt")
        if jgh.getDate('e').year != jahr:
            raise EE("Jahr des Endes der Beratung nicht gleich dem Jahr")
        # Soll verhindern, dass man ey und jahr gemeinsam nach dem Fallabschluss verschiebt.
        if fall and fall['zday'] > 0 and jahr > fall['zday']:
            raise EE("Fehler im Jahr: Jahr fr abgeschlossene Flle darf nicht nach dem Jahr des Fallabschlusses sein")
    # Beratungskontakte direkt abhandeln sind schon abgehandelt
    if fall:
        fall['akte'].akte_undo_cached_fields()

def _jgh07_check_abschnitt_K(form, jgh):
    item_K = "K / Grnde fr die Hilfegewhrung"
    m1 = "Mehr als ein Hauptgrund"
    m2 = "Mehr als ein 2. Grund"
    m3 = "Mehr als ein 3. Grund"
    k1 = "Kein Hauptgrund"
    k2 = "Kein 2. Grund"
    v = "Hauptgrund, 2. Grund und 3. Grund mssen unterschiedlich sein"
    found = True # falls das auf False gesetzt wird und danach wieder auf True --> Fehler
    for g, m in zip(('gr1', 'gr2', 'gr3'), (m1, m2, m3)):
        g_val = form.get(g)
        if isinstance(g_val, list):
            if len(g_val) > 1:
                raise EE("%s in %s" % (m, item_K))
            if g_val:
                g_val = g_val[0]
        if g_val:
            if not found:
                raise EE("%s in %s" % (k2, item_K))
            jgh[g] = check_code(form, g, 'gruende', "Fehler in %s" % item_K)
            found = jgh[g]
        else:
            if g == 'gr1':
                raise EE("%s in %s" % (k1, item_K))
            found = False
    v1, v2, v3 = jgh.get('gr1'), jgh.get('gr2'), jgh.get('gr3')
    if (v1 and (v1 == v2 or v1 == v3)) or (v2 and v2 == v3):
        raise EE("%s in %s" % (v, item_K))
    

def jgh07einf(form):
    """Jugendhilfestatistik 2007."""
    
    jghid = check_int_not_empty(form, 'jghid', "JugendhilfestatistikID fehlt")
    check_not_exists(jghid, Jugendhilfestatistik2007,
        "Jugendhilfestatistik (id: %(id)s, Fallnummer: %(fall_fn)s, Mitarbeiter: %(mit_id__na)s, Fall: %(fall_id)s ) existiert bereits")
    jgh = Jugendhilfestatistik2007()
    # Zur Einfhrung muss ein Fall vorhanden sein.
    # Evt. kann dieser spter gelscht werden, ohne dass die Statistik
    # selbst gelscht wird.
    # Der Fall (fall_id und fall_fn) kann spter nicht mehr verndert werden.
    jgh['fall_id'] = check_fk(form, 'fallid', Fall, "Kein Fall")
    jgh['fall_fn'] = check_str_not_empty(form, 'fall_fn',
                                         "Fallnummer fehlt")
    check_unique(jgh['fall_fn'], Jugendhilfestatistik2007List,
                 'fall_fn',
                 "Jugendhilfestatistik fr Fallnummer '%s'" % jgh['fall_fn']
                  + " existiert bereits.")
    check_unique(jgh['fall_id'], Jugendhilfestatistik2007List,
                 'fall_id',
                 "Jugendhilfestatistik fr Fall-ID '%s'" % jgh['fall_id']
                 + " existiert bereits.")
##     print 'EBUPD VOR _jgh07_check form', form
##     print 'EBUPD VOR _jgh07_check jgh', jgh
    _jgh07_check(form, jgh)
##     print 'EBUPD NACH _jgh07_check form', form
##     print 'EBUPD NACH _jgh07_check jgh', jgh
    jgh['zeit'] = int(time.time())
    try:
        jgh.insert(jghid)
        #print 'EBUPD INSERT erfolgreich'
        from ebkus.db.sql import SQL
        #print SQL("select * from jghstat07 order by ey").execute()
    except Exception, args:
        try: jgh.delete()
        except: pass
        raise EBUpdateError("Fehler beim Einfgen in die Datenbank: %s" % str(args))
        


def _jgh_get_letzte_laufende_nummer():
    letzte = letzte2007 = 0
    jghstatliste2007 = Jugendhilfestatistik2007List(where = 'lnr > 0', order = 'lnr')
    jghstatliste = JugendhilfestatistikList(where = 'lnr > 0', order = 'lnr')
    if jghstatliste:
        letzte = jghstatliste[-1]['lnr']
    if jghstatliste2007:
        letzte2007 = jghstatliste2007[-1]['lnr']
    return max(letzte, letzte2007)

def jgh_laufende_nummer_setzen():
    """Erteilt jeder Jugendhilfestatistik eine laufende Nummer.
    Im Prinzip kann in einer alten Statistik eine hhere Nummer stehen
    als in einer neuen, was aber selten vorkommen drfte.
    """

    letzte = _jgh_get_letzte_laufende_nummer()
    jghstatliste = JugendhilfestatistikList(where = 'lnr IS NULL', order = 'id')
    for j in jghstatliste:
        letzte += 1
        j.update({'lnr': letzte})
    jghstatliste = Jugendhilfestatistik2007List(where = 'lnr IS NULL', order = 'id')
    for j in jghstatliste:
        letzte += 1
        j.update({'lnr': letzte})
    return letzte

def upgrade_jgh(id, old2new):
    """Alte/neue Jugendhilfestatistik durch neue/alte ersetzen.
    Eine vorhandene Jugendstatistik wird durch die alte bzw. neue
    ersetzt, die vom Anwender ausgefllt werden muss.
    Die neue wird mit jgheinf bzw. jgh07einf angelegt.
    Falls old2new True ist, wird eine alte Bundesstatistik (bis 2006) durch
    eine neue (ab 2007) ersetzt, sonst umgekehrt.

    Das ganze kann ab 2008 auf jeden Fall gestrichen werden, da dann solche
    Flle nicht vorkommen drften.
    """
    logging.debug("upgrade_jgh: id=%s old2new=%s" % (id,old2new))
    try:
        if old2new:
            jgh = Jugendhilfestatistik(id)
        else:
            jgh = Jugendhilfestatistik2007(id)
    except:
        # nichts zu upgraden
        return
    fall_id = jgh.get('fall_id')
    fall = None
    if fall_id:
        fall = Fall(fall_id)
        if not fall['aktuell']:
            adj = old2new and 'neue' or 'alte'
            raise EE(
                "Ersatz duch %s Bundesstatistik nur mglich fr offene Flle." %
                adj)
        fall_beginn = fall.getDate('bg')
        if not fall_beginn.year <= 2006:
            raise EE("Ersatz duch %s Bundesstatistik nur mglich \
fr Flle von 2006 oder frher." % adj)
    else:
        raise EE(
            "Kein Fall vorhanden. Ersatz durch %s  Bundesstatistik nicht mglich." %
            adj)
    jgh.delete()
    logging.info("upgrade_jgh: %s Jugendhilfestatistik fr Fall-Nr. %s gelscht" %
                 (old2new, fall['fn']))

def updjgh07(form):
    """Update der Jugendhilfestatistik."""
    jghold = check_exists(form, 'jghid', Jugendhilfestatistik2007,
                              "JugendhilfestatistikID fehlt")
    jgh = Jugendhilfestatistik2007()
    _jgh07_check(form, jgh)
    jgh['zeit'] = int(time.time())
    jghold.update(jgh)
    try:
        ## akte_undo_cached_fields() fuehrt zu einem Error, wenn
        ## jgh['fall_id'] = None ist. Daher try-except
        _stamp_akte(jghold['akte'])
    except:
        # kann schief gehen, wenn es keinen Fall gibt
        pass
        


def upd_or_einf_abfr(form):
    from ebkus.html.abfragedef import Query
    abfr_id =  form.get('abfrid')
    name=check_str_not_empty(form, 'name', "Kein Name")
    abfrage_list = AbfrageList(where="name = '%s'" % name)
    if len(abfrage_list) > 1:
        raise EE(("Es existieren mehrere Teilmengendefinitionen mit dem Namen '%s'." % name) +
                 "Bitte erst lschen.")
    elif abfrage_list:
        abfrold = abfrage_list[0]
        if abfr_id and int(abfr_id) != abfrold['id']:
            raise EE("Es existiert bereits eine Definition mit dem Namen '%s'." % name)
    else:
        abfrold = None
    mit_id = check_fk(form, 'mitid', Mitarbeiter, "Kein Mitarbeiter")
    cgi_name = 'query1' 
    query1 = check_list(form, cgi_name, 'Keine Query', [])
    q = Query(query1, cgi_name=cgi_name)
    abfr = Abfrage()
    abfr.init(zeit=time.time(),
              mit_id=mit_id,
              name=name,
              dok=form.get('dok', ''),
              value=q.get_query_string(),
              typ='statistik_teilmengen_definition',
              )
    if abfrold:
        abfrold.update(abfr)
    else:
        abfr.new()
        abfr.insert()

## def updabfr(form):
##     from ebkus.html.abfragedef import Query
##     abfr_id =  check_int_not_empty(form, 'abfrid', "ID fehlt")
##     abfrold = Abfrage(abfr_id)
##     name=check_str_not_empty(form, 'name', "Kein Name")
##     mit_id = check_fk(form, 'mitid', Mitarbeiter, "Kein Mitarbeiter")
##     cgi_name = 'query1' 
##     q = Query(cgi_name, form.get(cgi_name, []))
##     abfr = Abfrage()
##     abfr.init(id=abfr_id,
##               zeit=time.time(),
##               mit_id=mit_id,
##               name=name,
##               dok=self.form.get('dok', ''),
##               value=q.get_query_string(),
##               typ='statistik_teilmengen_definition',
##               )
##     abfrold.update(abfr)

def rmabfr(form):
    abfr_id =  check_int_not_empty(form, 'abfrid', "ID fehlt")
    abfrold = Abfrage(abfr_id)
    abfrold.delete()
        
def gruppeeinf(form):
    """Neue Gruppe."""
    
    gruppeid = check_int_not_empty(form, 'gruppeid', "Gruppeid fehlt")
    check_not_exists(gruppeid, Gruppe,
                     "Gruppe (id: %(id)s, Name: %(name)s gibt es in der Datenbank bereits")
    gruppe = Gruppe()
    get_string_fields_new(gruppe, form, ['name', 'thema'], check_empty=True)
    gruppe['teiln'] = check_code(form, 'teiln', 'teiln',
                                 "Fehler bei Teilnehmer")
    gruppe['grtyp'] = check_code(form, 'grtyp', 'grtyp',
                                 "Fehler beim Gruppentyp")
    gruppe['stz'] = check_code(form, 'stz', 'stzei',
                               "Kein Stellenzeichen fr die Grupp")
    get_int_fields(gruppe, form, ['tzahl', 'stzahl'], None)
    
##     if form.has_key('mitid'):
##         mitid = form.get('mitid')
##     else: raise EE("Keine Mitarbeiter angegeben")
##     #  print "Mitarbeiter:",mitid
##     if type(mitid) is type(''):
##         mitid = [mitid]
    mitid = check_list(form, 'mitid', "Keine Mitarbeiter angegeben")
    # TBD: stimmt das? drfen beide Daten 0 sein oder in der Zukunft liegen?
    gruppe.setDate('bg', check_date(form, 'bg',
                                    "Fehler im Datum fr den Beginn", Date(0,0,0), maybezero = 1, maybefuture=1 ))
    gruppe.setDate('e', check_date(form, 'e',
                                   "Fehler im Datum fr das Ende", Date(0,0,0), maybezero = 1, maybefuture=1 ))
    if gruppe.getDate('bg') > gruppe.getDate('e'):
        raise EE('Endedatum liegt vor Beginndatum')
    gruppe['gn'] = getNewGruppennummer(Code(gruppe['stz'])['code'], gruppe['bgy'])
    check_unique(gruppe['gn'], GruppeList, 'gn',
                 "Gruppennummer ist bereits vergeben")
    for m in mitid:
        mitgruppe = MitarbeiterGruppe()
        mitgruppe['mit_id'] = int(m)
        mitgruppe['gruppe_id'] = gruppeid
        mitgruppe.setDate('bg', gruppe.getDate('bg'))
        mitgruppe.setDate('e', gruppe.getDate('e'))
        mitgruppe['zeit'] = int(time.time())
        try:
            mitgruppe.new()
            mitgruppe.insert()
        except Exception, args:
            raise EBUpdateError("Fehler beim Einfgen in die Datenbank: %s" % str(args))
            
    gruppe['zeit'] = int(time.time())
    
    try:
        gruppe.insert(gruppeid)
    except Exception, args:
        try: gruppe.delete()
        except: pass
        raise EBUpdateError("Fehler beim Einfgen in die Datenbank: %s" % str(args))
        
        
def updgr(form):
    """Update der Gruppe."""
    
    gruppeold = check_exists(form, 'gruppeid', Gruppe, "Gruppe-Id fehlt")
##     if form.has_key('mitid'):
##         mitid = form.get('mitid')
##     else:
##         raise EE("Keine Mitarbeiter angegeben")
##     if type(mitid) is type(''):
##         mitid = [mitid]
    mitid = check_list(form, 'mitid', "Keine Mitarbeiter angegeben")
    gruppe = Gruppe()
    get_string_fields_new(gruppe, form, ['gn', 'name', 'thema'], gruppeold, check_empty=True)
    gruppe['teiln'] = check_code(form, 'teiln', 'teiln',
                                 "Fehler bei Teilnehmer")
    gruppe['grtyp'] = check_code(form, 'grtyp', 'grtyp',
                                 "Fehler beim Gruppentyp")
    gruppe['stz'] = check_code(form, 'stz', 'stzei',
                               "Kein Stellenzeichen fr die Gruppe")
    get_int_fields(gruppe, form, ['tzahl', 'stzahl'], None)
    
    gruppe.setDate('bg', check_date(form, 'bg',
                                    "Fehler im Datum fr den Beginn",
                                    Date(0,0,0), maybezero = 1, maybefuture=1 ))
    gruppe.setDate('e', check_date(form, 'e',
                                   "Fehler im Datum fr das Ende",
                                   Date(0,0,0), maybezero = 1, maybefuture=1 ))
    
    if gruppe.getDate('bg') > gruppe.getDate('e'):
        raise EE('Beginndatum liegt vor Endedatum')
    gruppe['zeit'] = int(time.time())
    
    ##*************************************************************************
    ##
    ##  MastaleckT 07.03.2002
    ##
    ##  Update der Mitarbeiter einer Gruppe
    ##
    ##*************************************************************************
    
    ## Alle alten Mitarbeiter der Gruppe loeschen
    gr_mit_hinzuliste = []
    grp_mitarbeiterliste = MitarbeiterGruppeList(where = 'gruppe_id=%d' %gruppeold['id'])
    for mloesch in grp_mitarbeiterliste:
        mloesch.delete()
    for m in mitid:
        mitgruppe = MitarbeiterGruppe()
        mitgruppe['mit_id'] = int(m)
        mitgruppe['gruppe_id'] = gruppeold['id']
        ## Sollte unterschieden werden bei Mitzustndigkeit spter als Gruppenbeginn?
        mitgruppe.setDate('bg', gruppeold.getDate('bg'))
        mitgruppe.setDate('e', gruppeold.getDate('e'))
        mitgruppe['zeit'] = int(time.time())
        try:
            mitgruppe.new()
            mitgruppe.insert()
        except Exception, args:
            raise EBUpdateError("Fehler beim Einfgen in die Datenbank: %s" % str(args))
    try:
        gruppeold.update(gruppe)
    except Exception, args:
        try: gruppe.delete()
        except: pass
        raise EBUpdateError("Fehler beim Einfgen in die Datenbank: %s" % str(args))
    gruppeold.gruppe_undo_cached_fields()
    
    
def dokgreinf(form):
    """Gruppendokument (Text) einfgen."""
    
    import os
    dokid = check_int_not_empty(form, 'dokid', "dokid fehlt")
    check_not_exists(dokid, Gruppendokument,
      "Dokument-ID (id: %(id)s) existiert bereits")
    dok = Gruppendokument()
    dok['gruppe_id'] = check_fk(form, 'gruppeid', Gruppe, "Keine Gruppe")
    gruppeold = Gruppe(dok['gruppe_id'])
    dok['mit_id'] = check_fk(form, 'mitid', Mitarbeiter, "Kein Mitarbeiter")
    get_string_fields_new(dok, form, ['betr'], check_empty=True)
    dok['art'] = check_code(form, 'art', 'dokart', "Text ist: fehlt")
    dok['mtyp'] = cc('mimetyp','txt')
    dok['fname'] = '%s.txt' % dokid
    dok['zeit'] = int(time.time())
    dok.setDate('v',
                  check_date(form, 'v',
                             "Fehler im Datum"))
    text = check_str_not_empty(form, 'text', "Kein Text")
    gruppe_path = mk_gruppe_dir(gruppeold['id'])
    
    try:
        f = open('%s/%s' % (gruppe_path, dok['fname']), 'w')
        f.write(text)
        f.close()
        gruppe_path = get_gruppe_path(gruppeold['id'])
        os.chmod('%s/%s' % (gruppe_path, dok['fname']) ,0600)
    except Exception, args:
        raise EBUpdateError("Fehler beim Anlegen der Datei: %s" % str(args))
        
    gruppe = Gruppe()
    gruppe['zeit'] = int(time.time())
    
    try:
        dok.insert(dokid)
        gruppeold.update(gruppe)
    except Exception, args:
        try: dok.delete()
        except: pass
        raise EBUpdateError("Fehler beim Einfgen in die Datenbank: %s" % str(args))
        
    dok['gruppe'].gruppe_undo_cached_fields()
    
    
def updgrvermeinf(form):
    """Update der Gruppe."""
    
    import os
    dokold = check_exists(form, 'dokid', Gruppendokument,
                          "Dokument-Id fehlt")
    dok = Gruppendokument()
    dok['gruppe_id'] = check_fk(form, 'gruppeid', Gruppe,
                                "Keine Gruppe", dokold)
    dok['mit_id'] = check_fk(form, 'mitid', Mitarbeiter,
                             "Kein Mitarbeiter", dokold)
    get_string_fields_new(dok, form, ['betr'], dokold, check_empty=True)
    dok['art'] = check_code(form, 'art', 'dokart',
                            "Text ist: fehlt", dokold)
    dok.setDate('v',
                  check_date(form, 'v',
                             "Fehler im Datum"))
    dok['zeit'] = int(time.time())
    gruppeold = Gruppe(dok['gruppe_id'])
    if not is_binary(dokold):
        dok['mtyp'] = cc('mimetyp','txt')
        dok['fname'] = '%s.txt' % dokold['id']
        text = check_str_not_empty(form, 'text', "Kein Text")
        gruppe_path = get_gruppe_path(gruppeold['id'])
        fname = os.path.join(gruppe_path, dok['fname'])
        try:
            f = open(fname, 'w+')
            f.write(text)
            f.close()
            os.chmod(fname,0600)
        except Exception, args:
            raise EBUpdateError("Fehler beim Anlegen der Datei: %s" % str(args))
        
    dokold.update(dok)
    gruppe = Gruppe()
    gruppe['zeit'] = int(time.time())
    gruppeold.update(gruppe)
    dokold['gruppe'].gruppe_undo_cached_fields()
    
    
def uploadgreinf(form):
    """Upload eines Gruppendokumentes."""
    
    #  import mimetypes
    import os
    
    if form.has_key('datei'):
        dokid = check_int_not_empty(form, 'dokid', "Dokumentenid fehlt")
        check_not_exists(dokid, Gruppendokument,
        "Dokument (id: %(id)s) existiert bereits")
        dok = Gruppendokument()
        dok['gruppe_id'] = check_fk(form, 'gruppeid', Gruppe, "Keine Gruppe")
        gruppeold = Gruppe(dok['gruppe_id'])
        dok['mit_id'] = check_fk(form, 'mitid', Mitarbeiter, "Kein Mitarbeiter")
        dok['betr'] = check_str_not_empty(form, 'betr', "Kein Betreff")
        get_string_fields_new(dok, form, ['betr'], check_empty=True)
        dok['art'] = check_code(form, 'art', 'dokart', "Text ist: fehlt")
        dok.setDate('v',
                      check_date(form, 'v',
                               "Fehler im Datum"))
        dok['zeit'] = int(time.time())
        
        try:
            headers = form['datei'].headers
            if headers.has_key('Content-Type'):
                ctype = headers['Content-Type']
        except Exception, args:
            raise EE("Keine Dateiheaders erhalten: %s" % str(args))
            
            # Netscape sendet: Content-Type: application/xxx
            # Opera sendet: Content-Type: application/xxx; name="filename.ext"
            # Der Mime Typ wird hier nur von der .ext abgeleitetet !
            
        try:
            fname_orig = form['datei'].filename
        except Exception, args:
            raise EE("Kein Dateiname erhalten: %s" % str(args))
            
        try:
            i = string.rindex(fname_orig, '.')
            ext = fname_orig[i+1:]
            dok['mtyp'] = cc('mimetyp', '%s' % ext)
            ##      content_type = form['datei'].headers['Content-Type']
            ##      dok['mtyp'] = cn('mimetyp', content_type)
            dok['fname'] = '%s.%s' % (dokid, ext)
        except Exception, args:
            raise EE("Kein passendes Dateiformat (Mime Typ). %s" % str(args))
            
        try:
            f = open('%s/%s' % (mk_gruppe_dir(gruppeold['id']), dok['fname']), 'wb')
        except Exception, args:
            raise EBUpdateError("Fehler beim Oeffnen der Datei. %s" % str(args))
        try:
            f.write(form['datei'].read())
        except Exception, args:
            raise EBUpdateError("Fehler beim Speichern der Datei. %s" % str(args))
        try:
            f.close()
        except Exception, args:
            raise EBUpdateError("Fehler beim Schliessen der Datei. %s" % str(args))
        try:
            gruppe_path = get_gruppe_path(gruppeold['id'])
        except Exception, args:
            raise EBUpdateError("Fehler beim Finden des Dateipfades. %s" % str(args))
        try:
            os.chmod('%s/%s' % (gruppe_path, dok['fname']) ,0600)
        except Exception, args:
            raise EBUpdateError("Fehler beim Setzen der Dateirechte: %s" % str(args))
            
        gruppe = Gruppe()
        gruppe['zeit'] = int(time.time())
        
        try:
            dok.insert(dokid)
            gruppeold.update(gruppe)
        except Exception, args:
            try: dok.delete()
            except: pass
            raise EBUpdateError("Fehler beim Einfgen in die Datenbank: %s" % str(args))
            
        dok['gruppe'].gruppe_undo_cached_fields()
        
        
def gruppeteilneinf(form):
    """Neuer Gruppenteilnehmer."""
    
    mitid = check_int_not_empty(form, 'mitid', "Mitarbeiterid fehlt")
    gruppeid = check_int_not_empty(form, 'gruppeid', "Gruppeid fehlt")
    gruppeold = Gruppe(gruppeid)
    
    fallid = check_list(form, 'fallid', '', [])
    bezugspid = check_list(form, 'bezugspid', '', [])
    if not fallid and not bezugspid:
        raise EE("Keine TeilnehmerIn angegeben")
    for f in fallid:
        fallgruppel = FallGruppeList(where = 'gruppe_id = %s and fall_id = %s'
                                     % (gruppeid, f))
        if len(fallgruppel) > 0:
            # das muss nicht gemeldet werden
            #raise EE("Klient wird schon als Teilnehmer gefhrt")
            continue
        fallgruppe = FallGruppe()
        fallgruppe['fall_id'] = f
        fallgruppe['gruppe_id'] = gruppeid
        fallgruppe.setDate('bg', check_date(form, 'bg',
            "Fehler im Datum fr den Beginn", Date(0,0,0), maybezero = 1, maybefuture = 1 ))
        fallgruppe.setDate('e', check_date(form, 'e',
          "Fehler im Datum fr das Ende", Date(0,0,0), maybezero = 1, maybefuture = 1 ))
        if fallgruppe.getDate('bg') > fallgruppe.getDate('e'):
            raise EE('Beginndatum liegt vor Endedatum')

        fallgruppe['zeit'] = int(time.time())
        try:
            fallgruppe.new()
            fallgruppe.insert()
        except Exception, args:
            raise EBUpdateError("Fehler beim Einfgen in die Datenbank: %s" % str(args))
                
    for b in bezugspid:
        bezugspgruppel = BezugspersonGruppeList(where =                                               'gruppe_id = %s and bezugsp_id = %s' % (gruppeid, b))
        if len(bezugspgruppel) > 0:
            # das muss nicht gemeldet werden
            #raise EE("Bezugsperson wird schon als Teilnehmer gefhrt")
            continue
        bezugspgruppe = BezugspersonGruppe()
        bezugspgruppe['bezugsp_id'] = b
        bezugspgruppe['gruppe_id'] = gruppeid
        bezugspgruppe.setDate('bg', check_date(form, 'bg',
           "Fehler im Datum fr den Beginn", Date(0,0,0), maybezero = 1 ))
        bezugspgruppe.setDate('e', check_date(form, 'e',
           "Fehler im Datum fr das Ende", Date(0,0,0), maybezero = 1 ))
        bezugspgruppe['zeit'] = int(time.time())
        if bezugspgruppe.getDate('bg') > bezugspgruppe.getDate('e'):
            raise EE('Beginndatum liegt vor Endedatum')
        try:
            bezugspgruppe.new()
            bezugspgruppe.insert()
        except Exception, args:
            raise EBUpdateError("Fehler beim Einfgen in die Datenbank: %s" % str(args))
                
    gruppe = Gruppe()
    gruppe['zeit'] = int(time.time())
    gruppeold.update(gruppe)
    
    gruppeold.gruppe_undo_cached_fields()
    
    
def removeteiln(form):
    """Gruppenteilnehmer lschen."""
    
    gruppeid = check_int_not_empty(form, 'gruppeid', "Gruppe-Id fehlt")
    fallgr_id = check_list(form, 'fallgrid', '', [])
    bzpgr_id = check_list(form, 'bzpgrid', '', [])
    if not (fallgr_id or bzpgr_id):
        raise EE("Keine TeilnehmerIn zum Lschen markiert")
    for f in fallgr_id:
        fallgr = FallGruppe(f)
        fallgr.delete()
    for b in bzpgr_id:
        bzpgr = BezugspersonGruppe(b)
        bzpgr.delete()
    gruppeold = Gruppe(gruppeid)
    gruppe = Gruppe()
    gruppe['zeit'] = int(time.time())
    gruppeold.update(gruppe)

## def removeteiln(form):
##     """Gruppenteilnehmer lschen."""
    
##     gruppeid = check_int_not_empty(form, 'gruppeid', "Gruppe-Id fehlt")
##     fallgr_id = form.get('fallgrid')
##     bzpgr_id = form.get('bzpgrid')
##     if not (fallgr_id or bzpgr_id):
##         raise EE("Keine TeilnehmerIn zum Lschen markiert")
##     if fallgr_id:
##         if type(fallgr_id) is type(''):
##             fallgr_id = [fallgr_id]
##         for f in fallgr_id:
##             try:
##                 fallgr = FallGruppe(f)
##                 fallgr.delete()
##             except Exception, args:
##                 raise EBUpdateError("Fehler beim Lschen in der Datenbank: %s" % str(args))
                
##     if bzpgr_id:
##         if type(bzpgr_id) is type(''):
##             bzpgr_id = [bzpgr_id]
##         for b in bzpgr_id:
##             try:
##                 bzpgr = BezugspersonGruppe(b)
##                 bzpgr.delete()
##             except Exception, args:
##                 raise EBUpdateError("Fehler beim Lschen in der Datenbank: %s" % str(args))
                
##     gruppeold = Gruppe(gruppeid)
##     gruppe = Gruppe()
##     gruppe['zeit'] = int(time.time())
    
##     gruppeold.update(gruppe)
    
    
def updgrteiln(form):
    """Update des Gruppenteilnehmers."""
    
    if form.has_key('fallgrid'):
        fallgrold = check_exists(form, 'fallgrid', FallGruppe, "Kein Teilnehmer")
        gruppeold = fallgrold['gruppe']
        fallgr = FallGruppe()
        fallgr.setDate('bg', check_date(form, 'bg',
                                        "Fehler im Datum fr den Beginn",
                                        (0,0,0), # Datum darf leer sein
                                        maybezero = 1, maybefuture = 1 ))
        fallgr.setDate('e', check_date(form, 'e',
                                       "Fehler im Datum fr das Ende",
                                       (0,0,0), maybezero = 1,
                                       maybefuture = 1 ))
        fallgr['zeit'] = int(time.time())
        if fallgr.getDate('bg') > fallgr.getDate('e'):
            raise EE('Beginndatum liegt vor Endedatum')
        fallgrold.update(fallgr)
        gruppe = Gruppe()
        gruppe['zeit'] = int(time.time())
        gruppeold.update(gruppe)
        gruppeold.gruppe_undo_cached_fields()
        
    elif form.has_key('bzpgrid'):
        bezpgrold = check_exists(form, 'bzpgrid',
                                 BezugspersonGruppe, "Kein Teilnehmer")
        bezp = Bezugsperson(bezpgrold['bezugsp_id'])
        gruppeold = bezpgrold['gruppe']
        bezpgr = BezugspersonGruppe()
        bezpgr.setDate('bg', check_date(form, 'bg',
                                        "Fehler im Datum fr den Beginn",
                                        (0,0,0), maybezero = 1,
                                        maybefuture = 1 ))
        bezpgr.setDate('e', check_date(form, 'e',
                                       "Fehler im Datum fr das Ende",
                                       (0,0,0), maybezero = 1,
                                       maybefuture = 1 ))
        if bezpgr.getDate('bg') > bezpgr.getDate('e'):
            raise EE('Beginndatum liegt vor Endedatum')
        bezpgr['zeit'] = int(time.time())
        
        bezpgrold.update(bezpgr)
        gruppe = Gruppe()
        gruppe['zeit'] = int(time.time())
        gruppeold.update(gruppe)
        gruppeold.gruppe_undo_cached_fields()
    else:
        raise EE("keine Teilnehmer-ID erhalten.")
        
        
def removegrdoks(form):
    """Gruppendokument lschen."""
    
    import os
    dokids = check_list(form, 'dokids', "Keinen Eintrag markiert?")
    for d in dokids:
        try:
            dok = Gruppendokument(int(d))
            gruppeold = Gruppe(dok['gruppe_id'])
            gruppe_path = get_gruppe_path(gruppeold['id'])
            os.remove('%s/%s' % (gruppe_path, dok['fname']))
            gruppe = Gruppe()
            gruppe['zeit'] = int(time.time())
            gruppeold.update(gruppe)
            dok.delete()
        except Exception, args:
            raise EBUpdateError("Fehler beim Loeschen, id: %s" % str(args))
            
            #refresh des Cache?
            #Duplikat der Geloeschten Datensaetze und Dateien?
            #
            
            
def miteinf(form):
    """Neuer Mitarbeiter."""
    
    mitid = check_int_not_empty(form, 'mitid', "Mitarbeiterid fehlt")
    check_not_exists(mitid,
                     Mitarbeiter,
                     "Mitarbeiter (id: %(id)s, Vorname: %(vn)s, Name: %(na)s) gibt es in der DB bereits")
    
    mit = Mitarbeiter()
    get_string_fields_new(mit, form, ['vn', 'na', 'ben'], check_empty=True)
    mit['ben'] = check_unique(mit['ben'], MitarbeiterList, 'ben',
                      "Benutzername ist schon vorhanden. Muss einmalig sein")
    mit['stat'] = check_code(form, 'stat', 'status', "Fehler beim Status")
    mit['stz'] =  check_code(form, 'stz', 'stzei',
                             "Fehler beim Stellenzeichen")
    mit['benr'] =  check_code(form, 'benr', 'benr',
                              "Fehler bei den Benutzerrechten")
    mit['zeit'] = int(time.time())
    ##########################################################################################
    #
    # 13.09.2001 msg, TM
    #
    # Beim Anlegen eines Benutzers wird von nun an standardmaessig der benutzername als
    # default - Passwort in die Datenbank geschrieben. Die Benutzer muessen ihr Passwort
    # nach dem ersten Anmelden selbst aendern.
    # brehmea(msg) 13.09.2001 mit sha-encodierung
    ##########################################################################################
    s = sha.new(check_str_not_empty(form, 'ben', "Kein Benutzername"))
    mit['pass'] = s.hexdigest()
    
    try:
        mit.insert(mitid)
    except Exception, args:
        try: pers.delete()
        except: pass
        raise EBUpdateError("Fehler beim Einfgen in die Datenbank: %s" % str(args))
        
        
def updmit(form):
    """Update Mitarbeiter."""
    
    mitold = check_exists(form, 'mitid', Mitarbeiter, "Mitarbeiterid fehlt")
    mit = Mitarbeiter()
    
    get_string_fields_new(mit, form, ['vn', 'na', 'ben'], mitold, check_empty=True)
    if mitold['ben'] != mit['ben']:
        mit['ben'] = check_unique(mit['ben'], MitarbeiterList, 'ben',
                        "Benutzername ist schon vorhanden. Muss einmalig sein")
    mit['stat'] = check_code(form, 'stat', 'status',
                             "Fehler beim Status", mitold)
    mit['stz'] =  check_code(form, 'stz', 'stzei',
                             "Fehler beim Stellenzeichen", mitold)
    if mit['stz'] != mitold['stz']:
        from ebkus.app import ebapi
        for klassname in (
            'LeistungList',
            'MitarbeiterberatungskontaktList',
            'Fua_BSList',
            'ZustaendigkeitList',
            'DokumentList',
            'GruppendokumentList',
            'MitarbeiterGruppeList',
            'FachstatistikList',
            'JugendhilfestatistikList',
            'Jugendhilfestatistik2007List',
            'AbfrageList',
            ):
            res = getattr(ebapi, klassname)(where='mit_id = %(id)s' % mitold)
            if res:
                raise EE(
                    'Es sind bereits Daten mit diesem Mitarbeiter vorhanden.<br />'
                    'Ein Stellenwechsel ist nur mglich zur Fehlerkorrektur '
                    'direkt nachdem der Mitarbeiter in EBKuS angelegt wurde.')
    if (str(form.get('changepassword')) == '1'):
        s = sha.new(check_str_not_empty(form, 'ben', "Kein Benutzername"))
        mit['pass'] = s.hexdigest()
        
    mit['benr'] =  check_code(form, 'benr', 'benr',
                              "Fehler bei den Benutzerrechten", mitold)
    mit['zeit'] = int(time.time())
    
    mitold.update(mit)

    # So ist immer ein Fehler aufgetreten, wenn man einem Mitarbeiter eine
    # neu definierte Dienststelle zugeordnet hat und weiter Mitarbeiter hinzufgen
    # wollte.
    # undo_cached_fields()

    # Den ganzen Cache flushen beseitigt das Problem:
    cache_clear()
    
def codeeinf(form):
    """Neuer Code."""
    
    codeid = check_int_not_empty(form, 'codeid', "Merkmalsid fehlt")
    check_not_exists(codeid, Code,
                     "Code (id: %(id)s, Code: %(code)s, Kategoriencode: %(kat_code)s, Name: %(name)s) gibt es bereits")
    code = Code()
    get_string_fields_new(code, form, ['code', ('katcode', 'kat_code'), 'name'], check_empty=True)
    get_string_fields_new(code, form, ['dok'], '')
    try:
        c = Code(code=code['code'], kat_code=code['kat_code'])
        raise EBUpdateDataError("Code %(code)s existiert bereits" % c)
    except dbapp.DBAppError:
        # Ok, existiert nicht
        pass
    code['kat_id'] = check_fk(form, 'katid', Kategorie, 
                              "Kategorienid fehlt",
                              Kategorie(code=code['kat_code'])['id'])
    if form.get('mini') or form.get('maxi') or code['kat_code'] == 'dbsite':
        code['mini'] = check_int_not_empty(form, 'mini',
                                         "Fehler bei Bereichsminimum")
        code['maxi'] = check_int_not_empty(form, 'maxi',
                                         "Fehler bei Bereichsmaximum")
        if code['mini'] > code['maxi']:
            raise EE("Maximum ist kleiner als Minimum")
    else:
        code['mini'] = None
        code['maxi'] = None
    code['off'] = check_int_not_empty(form, 'off', 'Item off leer', 0)
    code['sort'] = check_int_not_empty(form, 'sort', "Sortierangabe fehlt")
    code['dm'] = None
    code['dy'] = None
    _check_code(code)
    codeliste = CodeList(where = "kat_code = '%s'" % code['kat_code']
                         + 'and sort >= %d' % code['sort'], order = 'sort')
    if codeliste:
    
        for c in codeliste:
            cneu = Code()
            x = c['sort']
            cneu['sort'] = x + 1
            c.update(cneu)
            
    code['zeit'] = int(time.time())
    
    try:
        code.insert(codeid)
    except Exception, args:
        try: code.delete()
        except: pass
        raise EBUpdateError("Fehler beim Einfgen in die Datenbank: %s" % str(args))
        

def _check_code(code):
    # kr 3-stellig
    kat_code = code['kat_code']
    code = code['code']
    if kat_code in ('kr', 'land', 'einrnr') and not code.isdigit():
        raise EE("Code muss aus Ziffern bestehen")
    if kat_code == 'kr' and len(code) != 3:
        raise EE("Code fr Kreis muss dreistellig sein (Ziffer 3-5 des amtlichen Gemeindeschlssel (AGS)")
    elif kat_code == 'land' and len(code) != 2:
        raise EE("Code fr Land muss zweistellig sein (Ziffer 1-2 des amtlichen Gemeindeschlssel (AGS)")
    elif kat_code == 'einrnr' and len(code) != 6:
        raise EE("Code fr Einrichtungsnummer muss sechsstellig sein")

def updcode(form):
    """Update des Code."""
    
    codeold = check_exists(form, 'codeid', Code, "Merkmalsid fehlt")
    code = Code()
    get_string_fields_new(code, form, ['code', ('katcode', 'kat_code'), 'name'], codeold, check_empty=True)
    get_string_fields_new(code, form, ['dok'], codeold)
    code['kat_id'] = check_fk(form, 'katid', Kategorie,
                              "Kategorienid fehlt", codeold['kat_id'])
    if form.get('mini') or form.get('maxi') or code['kat_code'] == 'dbsite':
        code['mini'] = check_int_not_empty(form, 'mini',
                                         "Minimum des Bereichs fehlt")
        code['maxi'] = check_int_not_empty(form, 'maxi',
                                         "Maximum des Bereichs fehlt")
        if code['mini'] > code['maxi']:
            raise EE("Minimum ist grsser als Maximum")
    else:
        code['mini'] = None
        code['maxi'] = None
    code['off'] = check_int_not_empty(form, 'off', 'Item off leer', 0)
    code['sort'] = check_int_not_empty(form, 'sort',
                                       "Sortierangabe fehlt", codeold)
    if form['dm'] != '':
        code['dm'] = check_int_not_empty(form, 'dm',
                                       "Fehler beim Ungltigkeitsdatum",)
        if code['dm'] < 1 or code['dm'] > 12:
            raise EE("Fehler beim Monat")
    else:
        code['dm'] = None
    if form['dy'] != '':
        code['dy'] = check_int_not_empty(form, 'dy',
                                       "Fehler beim Ungltigkeitsdatum", )
        if code['dy'] < 1000:
            raise EE("Keine 4 Ziffern beim Jahr")
    else:
        code['dy'] = None
        
    _check_code(code)
    x = code['sort']
    y = codeold['sort']
    
    code['zeit'] = int(time.time())
    codeold.update(code)
    
    # Korrektur der Sortierreihenfolge der Codes nach Aenderungen
    if x != y:
        codeliste = CodeList(where = "kat_id = %d" % codeold['kat_id']
                             + ' and sort = %d' % x , order = 'sort')
        for c in codeliste:
            if x > y and c['id'] != codeold['id']:
                cneu = Code()
                cneu['sort'] = c['sort'] - 1
                c.update(cneu)
            elif x < y  and c['id'] != codeold['id']:
                cneu = Code()
                cneu['sort'] = c['sort'] + 1
                c.update(cneu)
                
        codeliste = CodeList(where = "kat_id = %d" % codeold['kat_id'], order = 'sort')
        i = 0;
        for c in codeliste:
            i = i + 1
            cneu = Code()
            if c['sort'] == i:
                pass
            else:
                cneu['sort'] = i
                c.update(cneu)
    cache_clear()
            

def updkategorie(form):
    katold = check_exists(form, 'katid', Kategorie, "Kategorienid fehlt")
    kat = Kategorie()
    get_string_fields_new(kat, form, ['name'], katold, check_empty=True)
    get_string_fields_new(kat, form, ['dok'], katold)
    katold.update(kat)
    from ebkus.db.dbapp import undo_cached_fields
    undo_cached_fields()
    


def remove_akte(akte, statistik_auch=False, aktuell_auch=False):
    loeschmeldung = "Akte geloescht. Id: %(id)s Letzter Fall: %(letzter_fall__fn)s" % akte
    akte_id = akte['id']
    for fall in akte['faelle']:
        remove_fall(fall, statistik_auch, aktuell_auch)
    einrl = EinrichtungskontaktList(where = 'akte_id = %s' % akte_id)
    einrl.deleteall()
    bezugspl = BezugspersonList(where = 'akte_id = %s' % akte_id)
    for  b in bezugspl:
        bezugspgrl = BezugspersonGruppeList(where = 'bezugsp_id = %s' % b['id'])
        bezugspgrl.deleteall()
    bezugspl.deleteall()
    akte_path = get_akte_path(akte_id)
    try:
        os.rmdir('%s' % akte_path)
    except: pass
    akte.delete()
    undo_cached_fields()
    try:
        a = Akte(akte_id)
        assert False, 'Akte wurde nicht gelscht'
    except:
        pass # alles OK
    logging.info(loeschmeldung)
    return True

def remove_akten(akte_ids, statistik_auch=False, 
                 aktuell_auch=False, zda_vor=None):
    """
    akte_ids: liste oder tuple von int
    """
    existing_akte_ids = [akte['id'] for akte in AkteList(where_in=('akte.id', akte_ids))]
    if not existing_akte_ids:
        return
    else:
        akte_ids = existing_akte_ids
    # Haben offene Flle:
    if not aktuell_auch:
        akte_list = AkteList(where='fall.zday = 0',
                             where_in=('akte.id', akte_ids), 
                             join=[('fall', 'akte.id = fall.akte_id'),])
        if akte_list:
            raise EE("Akten mit offenen Flle knnen nicht gelscht werden")
    # zda nach zda_vor:
    if zda_vor is not None:
        akte_list = AkteList(
            where_in=('akte.id', akte_ids),
            where="CONCAT(LPAD(fall.zday, 4, '0'),LPAD(fall.zdam, 2, '0'),LPAD(fall.zdad, 2, '0')) > "
            "'{:04d}{:02d}{:02d}'".format(zda_vor.year, zda_vor.month, zda_vor.day), 
            join=[('fall', 'akte.id = fall.akte_id'),])
        if akte_list:
            raise EE("Akten, deren Abschluss nach dem {} liegt, knnen nicht gelscht werden".
                     format(zda_vor))
    # loeschmeldung = "Akte geloescht. Id: %(id)s Letzter Fall: %(letzter_fall__fn)s" % akte
    #loeschmeldung = "Akten geloescht: {}".format(len(akte_ids))
    loeschmeldung = "{} Akten geloescht mit den IDs: {}".format(len(akte_ids), ', '.join(str(x) for x in akte_ids))
    # Flle
    # fall_ids finden
    fall_ids = [f['id'] for f in FallList(where_in=('akte_id', akte_ids))]
    remove_faelle(fall_ids, statistik_auch=statistik_auch, aktuell_auch=aktuell_auch)
    # Einrichtungen
    einr_list = EinrichtungskontaktList(where_in=('akte_id', akte_ids))
    einr_list.deleteall()
    # Bezugsperson und BezugspersonGruppe
    bezugsp_list = BezugspersonList(where_in=('akte_id', akte_ids))
    bezugsp_ids = [b['id'] for b in bezugsp_list]
    bezugsp_gr_list = BezugspersonGruppeList(where_in=('bezugsp_id', bezugsp_ids))
    bezugsp_gr_list.deleteall()
    bezugsp_list.deleteall()
    # Dokumente (Dateien)
    for akte_id in akte_ids:
        akte_path = get_akte_path(akte_id)
        shutil.rmtree(akte_path, ignore_errors=True)
    # Schlielich die Akten selber
    AkteList(where_in=('id', akte_ids)).deleteall()
    logging.info(loeschmeldung)
    return len(akte_ids)

def remove_faelle(fall_ids, statistik_auch=False, aktuell_auch=False):
    #loeschmeldung = "Fall geloescht. Fallnummer: %s Beginn: %s Abschluss: %s" % \
    #    (fall['fn'], fall.getDate('bg'), fall.getDate('zda'))
    if not aktuell_auch:
        offene_faelle = FallList(where_in=('id', fall_ids), where='zday = 0')
        if offene_faelle:
            raise EE("Offene Flle knnen nicht gelscht werden")
    faelle = FallList(where_in=('id', fall_ids))
    # Daten fr das Loggen einsammeln
    fall_nummern = [f['fn'] for f in faelle]
    jghstatl = JugendhilfestatistikList(where_in=('fall_id', fall_ids))
    jgh07statl = Jugendhilfestatistik2007List(where_in=('fall_id', fall_ids))
    fachstatl = FachstatistikList(where_in=('fall_id', fall_ids))
    dokl = DokumentList(where_in=('fall_id', fall_ids))
    zustaendigl = ZustaendigkeitList(where_in=('fall_id', fall_ids))
    anmeldungl = AnmeldungList(where_in=('fall_id', fall_ids))
    leistungl = LeistungList(where_in=('fall_id', fall_ids))
    fallberl = FallberatungskontaktList(where_in=('fall_id', fall_ids))
    fallgrl = FallGruppeList(where_in=('fall_id', fall_ids))
    zbl = ZusaetzlicheBerechtigungList(where_in=('fall_id', fall_ids))
    wireb_list = WirebList(where_in=('fall_id', fall_ids))
    if statistik_auch:
        jghstatl.deleteall()
        jgh07statl.deleteall()
        fachstatl.deleteall()
    else:
        jghstatl.updateall({'fall_id': None})
        jgh07statl.updateall({'fall_id': None})
        fachstatl.updateall({'fall_id': None})
    anmeldungl.deleteall()
    leistungl.deleteall()
    dokl.deleteall()
    fallgrl.deleteall()
    zustaendigl.deleteall()
    zbl.deleteall()
    wireb_list.deleteall()
    # Beratungskontakte werden nur dann gelscht, wenn sie sich ausschlielich
    # auf den zu lschenden Fall beziehen. Ansonsten wird nur die Beteiligung
    # dieses Falls am Beratungskontakt gelscht.

    # FallberatungskontaktList lschen
    fallberl.deleteall()
    # Alle Kontakte finden, fr die nun keinen Fallberatungskontakt mehr haben
    bkont_list = BeratungskontaktList(where="NOT EXISTS (SELECT id from fallberatungskontakt where bkont_id = beratungskontakt.id)")
    bkont_ids = [b['id'] for b in bkont_list]
    # Diese knnen auch gelscht werden, sowie alle mitarbeiterberatungskontakte mit diesen Beratungskontakten
    bkont_list.deleteall()
    MitarbeiterberatungskontaktList(where_in=('bkont_id', bkont_ids)).deleteall()
    faelle.deleteall()
    loeschmeldung = "{} Faelle geloescht: {}".format(len(fall_nummern), ', '.join(fall_nummern))
    logging.info(loeschmeldung)
    return len(faelle)

def remove_fall(fall, statistik_auch=False, aktuell_auch=False):
    loeschmeldung = "Fall geloescht. Fallnummer: %s Beginn: %s Abschluss: %s" % \
        (fall['fn'], fall.getDate('bg'), fall.getDate('zda'))
    if not aktuell_auch:
        assert not fall['aktuell'], 'Aktueller Fall kann nicht gelscht werden'
    fall_id = fall['id']
    #print "********* LOESCHEN: %(id)s;%(akte_id)s;%(fn)s" % fall
    jghstatl = JugendhilfestatistikList(where = 'fall_id = %s' % fall_id)
    jgh07statl = Jugendhilfestatistik2007List(where = 'fall_id = %s' % fall_id)
    fachstatl = FachstatistikList(where = 'fall_id = %s' % fall_id)
    dokl = DokumentList(where = 'fall_id = %s' % fall_id)
    zustaendigl = ZustaendigkeitList(where = 'fall_id = %s' % fall_id)
    anmeldungl = AnmeldungList(where = 'fall_id = %s' % fall_id)
    leistungl = LeistungList(where = 'fall_id = %s' % fall_id)
    fallberl = FallberatungskontaktList(where='fall_id = %s' % fall_id)
    fallgrl = FallGruppeList(where = 'fall_id = %s' % fall_id)
    zbl = ZusaetzlicheBerechtigungList(where = 'fall_id = %s' % fall_id)
    akte_path = get_akte_path(fall['akte_id'])
    for j in jghstatl:
        if statistik_auch:
            j.delete()
        else:
            jghstatold = Jugendhilfestatistik(j['id'])
            jghstat = Jugendhilfestatistik()
            jghstat['fall_id'] = None
            jghstat['zeit'] = int(time.time())
            jghstatold.update(jghstat)
    for j in jgh07statl:
        if statistik_auch:
            j.delete()
        else:
            jghstatold = Jugendhilfestatistik2007(j['id'])
            jghstat = Jugendhilfestatistik2007()
            jghstat['fall_id'] = None
            jghstat['zeit'] = int(time.time())
            jghstatold.update(jghstat)
    for a in fachstatl:
        if statistik_auch:
            a.delete()
        else:
            fachstatold = Fachstatistik(a['id'])
            fachstat = Fachstatistik()
            fachstat['fall_id'] = None
            fachstat['zeit'] = int(time.time())
            fachstatold.update(fachstat)
    for d in dokl:
        try:
            os.remove('%s/%s' % (akte_path, d['fname']))
        except:
            pass
    anmeldungl.deleteall()
    leistungl.deleteall()
    dokl.deleteall()
    fallgrl.deleteall()
    zustaendigl.deleteall()
    zbl.deleteall()
    # Beratungskontakte werden nur dann gelscht, wenn sie sich ausschlielich
    # auf den zu lschenden Fall beziehen. Ansonsten wird nur die Beteiligung
    # dieses Falls am Beratungskontakt gelscht.
    for f in fallberl:
        bkont = f['bkont']
        # alle Beratungskontakte lschen, die sich *nur* auf diesen Fall beziehen
        if len(bkont['fallberatungskontakte']) == 1:
            assert f['id'] == bkont['fallberatungskontakte'][0]['id']
            # fr diese auch die Mitarbeiterberatungskontakte lschen
            bkont['mitarbeiterberatungskontakte'].deleteall()
            bkont.delete()
        try: f.delete()
        except: pass # wg. Fehler frherem Fehler kann er sich hier mal verschlucken
    fall.delete()
    logging.info(loeschmeldung)
    undo_cached_fields()
    return True

    

def remove_gruppe(gruppe):
    #print "%(id)s;;%(gn)s" % g
    loeschmeldung = "Gruppe geloescht. Name: %(name)s Gruppennummer: %(gn)s Mitarbeiter: %(mitarbeiternamen)s" % gruppe
    gruppe_id = gruppe['id']
    gruppe_path = get_gruppe_path(gruppe_id)
    fallgrl = FallGruppeList(where = 'gruppe_id = %s' % gruppe_id)
    bezugspgrl = BezugspersonGruppeList(where = 'gruppe_id = %s' % gruppe_id)
    dokl = GruppendokumentList(where = 'gruppe_id = %s' % gruppe_id)
    mitgrl = MitarbeiterGruppeList(where = 'gruppe_id = %s' % gruppe_id)
    fallgrl.deleteall()
    bezugspgrl.deleteall()
    mitgrl.deleteall()
    for d in dokl:
        try:
            os.remove('%s/%s' % (gruppe_path, d['fname']))
        except:
            pass
    try:
        os.rmdir('%s' % gruppe_path)
    except:
        pass
    dokl.deleteall()
    gruppe.delete()
    logging.info(loeschmeldung)
    return True
    
def removeakten(form):
    """Akten und Gruppen lschen."""
    akte_ids = check_list(form, 'rmak', 'Fehler in zu lschenden Akten', [])
    akte_ids = [int(x) for x in akte_ids]
    # Gruppen werden nach wie vor einzeln gelscht
    gruppen_ids = check_list(form, 'rmgr', 'Fehler in zu lschenden Gruppen', [])
    gruppen = [Gruppe(int(id)) for id in gruppen_ids]
    cacheon = cache_is_on()
    try:
        if cache_on:
            cache_off()
        remove_akten(akte_ids, statistik_auch=False, aktuell_auch=False)
        # for a in akten:
        #     remove_akte(a, statistik_auch=False)
        for g in gruppen:
            remove_gruppe(g)
    finally:
        if cacheon:
            cache_on()
    return len(akte_ids), len(gruppen)
    
def sperrakten(form):
    #print 'SPERRAKTEN', form, set, Akte('2')
    #print form.get('akten_ids')
    # print set([Akte('2'), Akte('8')])
    # akten = set([Akte(id) for id in form.get('akten_ids')])
    # zusperren = set([Akte(id) for id in form.get('akten_ids_gesperrt')])
    # zuentsperren = set([akte for akte in akten if akte not in zusperren])

    akten_ids = [int(id) for id in form.get('akten_ids')]
    akten_ids_checked = set([int(id) for id in form.get('akten_ids_gesperrt')])
    akten_ids_unchecked = [id for id in akten_ids if id not in akten_ids_checked]
    ja = cc('ja_nein', '1')
    nein = cc('ja_nein', '2')
    schon_gesperrt = [a['id'] for a in AkteList(where='sperre = %s' % ja, where_in=('id', akten_ids))]
    zusperren = [id for id in akten_ids_checked if id not in schon_gesperrt]
    zuentsperren = [id for id in akten_ids_unchecked if id in schon_gesperrt]
    AkteList(where_in=('id', zusperren)).updateall({'sperre': ja, 'zeit': int(time.time())})
    AkteList(where_in=('id', zuentsperren)).updateall({'sperre': nein, 'zeit': int(time.time())})
    return len(zusperren), len(zuentsperren)

    # akten = [Akte(id) for id in form.get('akten_ids')]
    # zusperren = [Akte(id) for id in form.get('akten_ids_gesperrt')]
    # zuentsperren = [akte for akte in akten if akte not in zusperren]

    # # Nur die updaten, deren Zustand sich tatschlich ndert
    # zusperren = [akte for akte in zusperren if not akte['gesperrt']]
    # zuentsperren = [akte for akte in zuentsperren if akte['gesperrt']]
    # for akte in zusperren:
    #     akte.update({'sperre': cc('ja_nein', '1')})
    #     _stamp_akte(akte)
    # for akte in zuentsperren:
    #     akte.update({'sperre': cc('ja_nein', '2')})
    #     _stamp_akte(akte)

def sperrakten_reset():
    """Das Feld 'sperre' von allen Akten auf NULL setzen.
    Damit sind alle Akten entsperrt und wieder fr die automatische Sperre zugnglich.
    """
    akten_list = AkteList(where = 'akte.sperre is not null')
    logging.info("Sperren zurueckgesetzt: %s Akten" % len(akten_list))
    akten_list.updateall({'sperre': None,'zeit': int(time.time())})
    return len(akten_list)

def sperrakten_automatisch():
    """Alle Akten sperren, die
    - keinen offenen Fall haben
    UND
    - seit dem letzten zdA sind mindestens volle SPERRFRIST Monate vergangen
    UND
    - SPERRFRIST >= 1
    UND
    - die nicht explizit entsperrt sind, dh akte['sperre'] == None bzw. sperre is null
      (Sonst wrde eine manuelle Entsperrung immer wieder rckgngig gemacht.)

    Nach jedem zdA wird das Feld 'sperre' auf None zurckgesetzt, so dass
    die automatische Sperre wieder greifen kann.
    """
    if config.SPERRFRIST <= 0:
        return
    logging.info("Automatisches Sperren. Sperrfrist: %s Monate" % config.SPERRFRIST)

    # Beispiel: Sperrfrist = 1, today = 2.2.2016 --> 2.1.2016
    # Beispiel: Sperrfrist = 1, today = 31.1.2016 --> 31.12.2016
    # Ein zdA-Datum gleich oder grer fhrt nicht zur Sperrung
    # Damit ist gewhrleistet, dass mindestens volle SPERRFRIST Monate nicht gesperrt wird.
    keine_sperre_ab_datum = today().add_month(- config.SPERRFRIST)
    y = keine_sperre_ab_datum.year
    m = keine_sperre_ab_datum.month
    zu_sperrende_akten = AkteList(
        where = 'akte.sperre is null and '
                'exists (select * from fall '
                '        where fall.akte_id = akte.id) and '
                'not exists (select * from fall '
                '            where fall.akte_id = akte.id and zday = 0) and '
                'not exists (select * from fall '
                '            where fall.akte_id = akte.id and '
                '                  (zday > %s or (zday = %s and zdam >= %s)))'
        % (y, y, m))
    zu_sperrende_akten.updateall({'sperre': cc('ja_nein', '1'),
                                  'zeit': int(time.time())})
    logging.info("Automatisches Sperren. %s Akten gesperrt" % len(zu_sperrende_akten))
            
def _stamp_akte(akteold):
    """Zeitstempel in der Akte setzen und cache zurcksetzen"""
    akte = Akte()
    akte['zeit'] = int(time.time())
    akteold.update(akte)
    akteold.akte_undo_cached_fields()
    


def setAdresse(obj, form):
    """Wenn strkat_on gesetzt ist, wird die Ueberpruefung durch den Strassenkatalog
    getriggert.

    Kann auch fuer Bezugspersonenadressen verwendet werden,
    plraum wird dann nicht beruecksichtigt
    """
    #print 'SETADRESSE', form
    from ebkus.html.strkat import fuehrende_nullen_ersetzen
    strkat_on = form.get('strkat_on')
    if strkat_on and config.STRASSENKATALOG:
        from ebkus.html.strkat import get_strassen_list
        try:
            strassen_list = get_strassen_list(form, exact=False)
            if strassen_list != None and len(strassen_list) > 1:
                strassen_list = get_strassen_list(form, exact=True)
        except Exception, e:
            raise EE(str(e))
        #print 'SETADRESSE', strassen_list
        if strassen_list == None:
            raise EE("Keine Adresse.<br /><br />"
                     "Falls Sie ausdrcklich keine Adresse angeben wollen, "
                     "deaktivieren Sie den Abgleich mit dem Straenkatalog.")
        elif len(strassen_list) < 1:
            raise EE("Kein passender Eintrag fr die Adresse "
                     "im Straenkatalog gefunden.<br /><br />"
                     "Bitte verwenden Sie die Straensuche,"
                     "oder deaktivieren Sie den Abgleich "
                     "mit dem Straenkatalog.""")
        elif len(strassen_list) > 1:
            raise EE("Mehrere passende Eintrge "
                     "im Straenkatalog gefunden.<br /><br />"
                     "Bitte whlen Sie mit der Straensuche "
                     "einen eindeutigen Eintrag aus.""")
        hsnr = form.get('hsnr')
        strasse = strassen_list[0]
        obj['lage'] = cc('lage', '0') 
        obj['ort'] = strasse['ort']
        obj['plz'] = strasse['plz']
        obj['str'] = strasse['name']
        von = strasse['von']
        if von and von == strasse['bis']:
            hsnr = fuehrende_nullen_ersetzen(von)
        if not hsnr:
            raise EE("Keine Hausnummer.<br /><br />"
                     "Falls Sie ausdrcklich keine Hausnummer angeben wollen, <br />"
                     "tragen Sie '---' anstelle der Hausnummer ein.")
        elif hsnr.startswith('-'):
            obj['hsnr'] = '' # erfolgreicher Abgleich ohne Hausnummer
        else:
            obj['hsnr'] = hsnr
        if 'plraum' in obj.fields:
            obj['plraum'] = strasse['plraum']
    else:
        # Ohne Abgleich mit Straenkatalog
        obj['lage'] = cc('lage', '1')
        obj['str'] = strs = form.get('str', '')
        if len(strs) > 35:
            raise EBUpdateDataError("Straenname zu lang: %s" % strs)
        obj['ort'] = ort = form.get('ort', '')
        if len(ort) > 35:
            raise EBUpdateDataError("Ortsname zu lang: %s" % ort)
        obj['hsnr'] = form.get('hsnr', '')
        if len(obj['hsnr']) > 5:
            raise EBUpdateDataError("Ungltige Hausnummer: %s" % obj['hsnr'])
        obj['plz'] = plz = form.get('plz', '')
        if len(plz) > 5 or (plz and not plz.isdigit()):
            raise EBUpdateDataError("Ungltige Postleitzahl: %s" % obj['plz'])
        if 'plraum' in obj.fields:
            planungs_raum = form.get('plraum')
            if planungs_raum in ('0',):
                raise EBUpdateDataError("Kein gltiger Planungsraum")
            obj['plraum'] =  planungs_raum or '0'

def konfig_validate(config, konfig_name, konfig_value, fachlich_only=False):
    """
    Prfen, ob der neue Wert konfig_value fr die Konfigurationsvariable
    konfig_name gltig ist. Wenn nicht, wird eine Exception geworfen.
    
    Wenn fachlich_only den Wert True hat, dann werden alle nicht-fachlichen Variablen
    als nicht-existent betrachtet auer today.
    
    Gibt das Konfigurationsobjekt zurck, eine Instanz von config.KonfigVar,
    die den alten Wert enthlt (kv.value).
    """
    # print 'KONFIG_VALIDATE', konfig_name, konfig_value
    # existiert name?
    konfig_list = [kv for kv in config.iter() if 
                   kv.name == konfig_name and 
                   (kv.fachlich or not fachlich_only or kv.name == 'today')]
    if konfig_list:
        kv = konfig_list[0]
    else:
        raise EE("Unbekannte Konfigurationsvariable: %s" % konfig_name)
    if kv.valid_pattern:
        m = re.match(kv.valid_pattern, konfig_value)
        if not m:
            raise EE("Ungltiger Wert fr '%s': %s" % (konfig_name, konfig_value))
    if kv.is_boolean():
        if konfig_value not in ('true', 'false'):
            raise EE("Ungltiger Wert fr boolsche Konfigurationsvariable: %s" % konfig_value)
    elif kv.is_int():
        try:
            konfig_value = int(konfig_value)
        except ValueError:
            raise EE("Ungltiger Wert fr integer Konfigurationsvariable: %s" % konfig_value)
    if konfig_name in ('zda_rueckgaengig_frist', 'wiederaufnahmefrist'):
        if konfig_name == 'zda_rueckgaengig_frist':
            zdaf = konfig_value
            waf = config.WIEDERAUFNAHMEFRIST
        else:
            zdaf = config.ZDA_RUECKGAENGIG_FRIST
            waf = konfig_value
        # print 'KONFIG_VALIDATE', zdaf, waf
        if zdaf != -1 and zdaf < waf:
            raise EE("zda_rueckgaengig_frist: {} Monate<br/>"
                     "wiederaufnahmefrist: {} Monate<br/>"
                     "Die zdA-Rckgngig-Frist muss mindestens so gro sein wie die Wiederaufnahmefrist. "
                     "Sonst entstnde eine Periode, in der weder ein zdA rckgngig gemacht werden kann, "
                     "noch eine Wiederaufnahme (Fortsetzung als neuer Fall)".format(zdaf, waf))
    return kv

def updkonfig(form):
    """Update Konfigurationsvariable."""
    # damit man auch nicht fachliche Variablen updaten kann (nur fr Tests)
    #logging.info('UPDKONFIG FORM: {0}'.format(form))
    #print 'UPDKONFIG', form
    all_vars = bool(check_str_not_empty(form, 'konfig_all', "", ''))
    konfig_name = check_str_not_empty(form, 'konfig_name', "Kein Name fr Konfigurationsvariable")
    konfig_value = form.get('konfig_value', '').strip()
    # kein Fehler fr today, ntig fr demo
    kv = konfig_validate(config, konfig_name, konfig_value, fachlich_only=not all_vars)
    register_set(konfig_name, konfig_value)
    if kv.is_boolean():
        konfig_value = konfig_value == 'true'
    elif kv.is_int():
        konfig_value = int(konfig_value)
    setattr(config, kv.NAME, konfig_value)
    if kv.NAME == 'GEMEINDESCHLUESSEL_VON_PLZ':
        from ebkus.app.gemeindeschluessel import delete_cache
        delete_cache()
    if kv.NAME == 'CACHE_ON':
        from ebkus.db.dbapp import set_cache
        set_cache(konfig_value)
    if kv.NAME == 'SPERRFRIST' and konfig_value == -1:
        sperrakten_reset()
    #logging.info('UPDKONFIG VAR VALUE: {0}: {1}'.format(kv.NAME, getattr(config, kv.NAME)))

# def updkonfig(form):
#     """Update Konfigurationsvariable."""
#     # damit man auch nicht fachliche Variablen updaten kann (nur fr Tests)
#     #logging.info('UPDKONFIG FORM: {0}'.format(form))
#     #print 'UPDKONFIG', form
#     all_vars = bool(check_str_not_empty(form, 'konfig_all', "", ''))
#     konfig_name = check_str_not_empty(form, 'konfig_name', "Kein Name fr Konfigurationsvariable")
#     # kein Fehler fr today, ntig fr demo
#     konfig_list = [kv for kv in config.iter() if kv.name == konfig_name and (kv.fachlich or all_vars or kv.name == 'today')]
#     if konfig_list:
#         kv = konfig_list[0]
#     else:
#         raise EE("Unbekannte Konfigurationsvariable: %s" % konfig_name)
#     konfig_value = form.get('konfig_value', '').strip()
#     if kv.valid_pattern:
#         m = re.match(kv.valid_pattern, konfig_value)
#         if not m:
#             raise EE("Ungltiger Wert fr '%s': %s" % (konfig_name, konfig_value))
#     if kv.is_boolean():
#         if konfig_value not in ('true', 'false'):
#             raise EE("Ungltiger Wert fr boolsche Konfigurationsvariable: %s" % konfig_value)
#     elif kv.is_int():
#         try:
#             int(konfig_value)
#         except ValueError:
#             raise EE("Ungltiger Wert fr integer Konfigurationsvariable: %s" % konfig_value)
#     register_set(konfig_name, konfig_value)
#     if kv.is_boolean():
#         konfig_value = konfig_value == 'true'
#     elif kv.is_int():
#         konfig_value = int(konfig_value)
#     setattr(config, kv.NAME, konfig_value)
#     if kv.NAME == 'GEMEINDESCHLUESSEL_VON_PLZ':
#         from ebkus.app.gemeindeschluessel import delete_cache
#         delete_cache()
#     if kv.NAME == 'CACHE_ON':
#         from ebkus.db.dbapp import set_cache
#         set_cache(konfig_value)
#     if kv.NAME == 'SPERRFRIST' and konfig_value == -1:
#         sperrakten_reset()
#     #logging.info('UPDKONFIG VAR VALUE: {0}: {1}'.format(kv.NAME, getattr(config, kv.NAME)))

def wirebeinf(form):
    """Neue Wireb-Anbindung"""
    wirebid = check_int_not_empty(form, 'wirebid', "Wireb-Id fehlt")
    check_not_exists(wirebid, Wireb, "Wireb (id: %(id)s) existiert bereits")
    wireb = Wireb()
    wireb['fall_id'] = check_fk(form, 'fallid', Fall, "Kein Fall")
    wireb.setDate('fcbg', today())
    fallcodenummer = check_int_not_empty(form, 'fallcodenummer', "Keine Fallcodenummer")
    wbl = WirebList(where='fallcodenummer = {}'.format(fallcodenummer))
    if wbl:
        raise EE("Fallcodenummer {} wird bereits fr Fall {} verwendet.".format(fallcodenummer, wbl[0]['fall__fn']))
    wireb['fallcodenummer'] = fallcodenummer
    try:
        wireb.insert(wirebid)
    except:
        try: wireb.delete()
        except: pass
    _stamp_akte(wireb['fall__akte'])

def updwireb(form):
    wirebold = check_exists(form, 'wirebid', Wireb, "Keine Wireb-Id")
    wireb = Wireb()
    #print 'UPDWIREB wirebid:', wirebold['id']
    #print form
    fallcodenummer = check_int_not_empty(form, 'Keine Fallcodenummer', 'fallcodenummer', '')
    #print 'UPDWIREB fallcodenummer:', fallcodenummer
    if fallcodenummer and fallcodenummer != wirebold['fallcodenummer']:
        wireb['fallcodenummer'] = fallcodenummer
        wireb.setDate('fcbg', today())
    jghbgy_new = check_int_not_empty(form, 'jghbgy', 'Kein Beginnjahr', -999)
    if jghbgy_new > 0:
        wireb.setDate('jghbg', check_date(form, 'jghbg', 'Fehler im Datum'))
    elif jghbgy_new == 0: # 0 bedeutet hier lschen, siehe wireb.py
        wireb['jghbgy'] = 0
        wireb['jghbgm'] = 0
        wireb['jghbgd'] = 0
    alle_bearbeitet = form.get('alle_bearbeitet')
    if alle_bearbeitet:
        alle_bearbeitet = check_code(form, 'alle_bearbeitet', 'ja_nein', "Fehler im Code fr alle_bearbeitet")
        wireb['alle_bearbeitet'] = alle_bearbeitet
    wirebold.update(wireb)
    _stamp_akte(wirebold['fall__akte'])
    
def removewireb(form):
    """Lschen der Wir.EB-Anbindung."""
    wireb = check_exists(form,'wirebid', Wireb, "Keine Wireb-Id")
    if wireb['jghbgy']:
        raise EE("Bitte erst die bereits nach Wir.EB hochgeladenen Daten der"
                 " Jugendhilfestatistik lschen!")
    wireb.delete()

# Braucht man vielleicht nicht?
# def _wireb_check(form):
#     pass

def zusaetzliche_mitarbeiter(form):
    # fallid
    # mitid (multiple)
    fall_id = check_fk(form, 'fallid', Fall, "Kein Fall")
    fall = Fall(fall_id)
    if not fall['aktuell']:
        raise EE('Zustzliche Berechtigung kann nur fr offende Flle vergeben werden.')
    mit_ids = check_list(form, 'mitid', 'Keine Mitarbeiter', default=[])
    zbl = ZusaetzlicheBerechtigungList(where='fall_id = {}'.format(fall_id))
    zbl.deleteall()
    for mit_id in mit_ids:
        zb = ZusaetzlicheBerechtigung()
        zb.init(
            mit_id=mit_id,
            fall_id=fall_id,
            )
        zb.new()
        zb.insert()
    _stamp_akte(fall['akte'])

