# coding: latin-1

##*************************************************************************
## Projekt:       EBKuS
##
## Datei:         jghstatistik.py
##
## Beschreibung:  ...
##
## Klassen:       jgh07neu
##                updjgh07
##*************************************************************************
##
## Revisionen:
##
## Datum:                 Autor:          Beschreibung:
##
##
##*************************************************************************

"""Module fr die Jugendhilfestatistik."""

import re
import os
import urllib
from ebkus.config import config
from ebkus.app import ebapi
from ebkus.app import Request
from ebkus.app.ebapi import Akte, Fall, Jugendhilfestatistik, Code, \
     JugendhilfestatistikList, cc, cn, today, EE, check_date, calc_age, \
     Jugendhilfestatistik2007List, Jugendhilfestatistik2007, \
     BezugspersonList, get_berichtseinheitid, get_object
from ebkus.app.ebupd import upgrade_jgh, updkonfig, konfig_validate
from ebkus.app.ebapih import get_codes, mksel, get_all_codes
from ebkus.app.gemeindeschluessel import get_gemeindeschluessel
from ebkus.app_surface.standard_templates import *
from ebkus.app_surface.jgh_templates import *
from ebkus.app_surface.jgh07_templates import *
import ebkus.html.htmlgen as h
from ebkus.html.akte_share import akte_share
from ebkus.app.jghdef import get_jghdef
        
def make_option_list(code_list, selected=None, no_empty_option=False):
    """Liefert eine Liste von Option-Elementen (Liste von Strings).
    
    selected entscheidet, welcher Wert selected ist.
    Falls selected die id eines Wertes in code_list ist, ist der selected.
    Falls selected None ist, ist kein Wert selected.
    Falls selected ' ' ist, ist die leere Option selected.
    Falls selected 'first' ist, ist der erste code selected.
    Falls add_empty False ist, wird keine leere Option eingefgt.
    """
    max_size = 60
    option = '\n<option value="%(id)s" %(sel)s >%(name)s</option>'
    res = []
    selected_values = ()

    if not no_empty_option:
        if selected in ('', ' '):
            res.append('<option value=" " selected> </option>')
        else:
            res.append('<option value=" "> </option>')
    if selected:
        if selected == 'first' and code_list:
            selected = code_list[0]['id']
        if not isinstance(selected, (tuple, list)):
            selected_values = (selected,)
    for c in code_list:
        sel = c['id'] in selected_values and 'selected' or ''
        c['sel'] = sel
        res.append(option % c)
        del c['sel']
    return res

# Obsolet durch neuen Bundesstatistikdefinition
# Zum Vergleich fr Berichtsjahr 2022 bleibt es noch drin
_fb_data = (
    {'abschnitt': 'A',
     'title': 'Beginn und Anlass der Hilfegewhrung',
     'items_data': ({'frage': 'Monat (der Einleitung der Hilfe)',
                     'typ': ('int', 1, 12,),
                     'name': 'bgm',
                     #'readonly': True,
                     },
                    {'frage': 'Jahr',
                     'typ': ('int', 1970, 2030,),
                     'name': 'bgy',
                     #'readonly': True,
                     },
                    {'frage': 'bernahme von einem anderen Jugendamt wegen Zustndigkeitswechsels',
                     'typ': ('checkbox','1', ' '),
                     'name': 'zustw',
                     },
                    
                    {'frage': 'Einleitung der Hilfe aufgrund einer vorangegangenen Gefhrdungseinschtzung ( 8a Absatz 1 SGB VIII)',
                     'typ': ('kat', 'ja_nein',),
                     'name': 'gef8a',
                     'default': ' '
                     },
                    # {'frage': '<br/><b>Nur unmittelbar nach unbegleiteter '
                    #           'Einreise aus dem Ausland</b><br/><br/> '
                    #           'Einleitung der Hilfe im Anschluss an eine Inobhutnahme nach  '
                    #           'unbegleiteter Einreise aus dem Ausland '
                    #           '( 42 Absatz 1 Satz 1 Nummer 3 SGB VIII)',
                    #  'typ': ('kat', 'ja_nein',),
                    #  'name': 'inobh42',
                    #  'default': ' '
                    #  },
                    {'frage': 'Einleitung der Hilfe im Anschluss an eine Inobhutnahme '
                              '(42 Absatz 1 Satz 1 SGB VIII)',
                     'typ': ('kat', 'inobh42',),
                     'name': 'inobh42_22',
                     'default': ' '
                     },
                    ),
     }, 
    {'abschnitt': 'B',
     'title': 'Art der Hilfe',
     'items_data': ({'frage': 'Art der Hilfe',
                'typ': ('kat', 'hilf_art',),
                'name': 'hilf_art',
                'default': ' '
                },
               ),
     }, 
    {'abschnitt': 'C',
     'title': 'Ort der Durchfhrung',
     'items_data': ({'frage': 'Ort an dem die Hilfe (hauptschlich) durchgefhrt wird',
                'typ': ('kat', 'hilf_ort',),
                'name': 'hilf_ort',
                'default': '06',
                'no_empty_option': True,     
                },
               ),
     },
    {'abschnitt': 'D',
     'title': 'Trger',
     'items_data': ({'frage': 'Trger der Einrichtung oder des Dienstes, die/der die Hilfe/Beratung durchfhrt',
                'typ': ('kat', 'traeger',),
                'name': 'traeger',
                'default': '10',
                'no_empty_option': True,     
                },
               ),
     },
    {'abschnitt': 'E',
     'title': 'Geschlecht, Geburtsmonat und -jahr',
     'items_data': (
            {'frage': 'Geschlecht, nach (Geburtenregister)',
             'typ': ('kat', 'gs',),
             'name': 'gs',
             'default': ' '
             },
            {'frage': 'Geburtsmonat',
             'typ': ('int', 1, 12,),
             'name': 'gem',
             'default': ' '
             },
            {'frage': 'Geburtsjahr',
             'typ': ('int', 1970, 2040,),
             'name': 'gey',
             'default': ' '
             },
            ),
     }, 
    {'abschnitt': 'F',
     'title': 'Lebenssituation bei Beginn der Hilfe',
     'items_data': ({'frage': 'Gewhnlicher Aufenthaltsort vor der Hilfe',
                'typ': ('kat', 'auf_ort',),
                'name': 'aort_vor',
                'default': ' '
                },
               {'frage': 'Situation in der Herkunftsfamilie',
                'typ': ('kat', 'shf',),
                'name': 'sit_fam',
                'default': ' '
                },
               {'frage': 'Auslndische Herkunft mindestens eines Elternteils (nicht: Staatsangehrigkeit)',
                'typ': ('kat', 'ja_ne_un',),
                'name': 'ausl_her',
                'default': ' '
                },
               {'frage': 'In der Familie wird vorrangig deutsch gesprochen',
                'typ': ('kat', 'ja_ne_un',),
                'name': 'vor_dt',
                'default': ' '
                },
               'Wirtschaftliche Situation',
               {'frage': 'Die Familie bzw. die/der junge Volljhrige lebt teilweise oder '
                         'ganz von <br/>'
                         '- Arbeitslosengeld (nach dem Zweiten Buch Sozialgesetzbuch - SGB II), '
                         'auch in Verbindung mit Sozialgeld, <br/> '
                         '- Sozialhilfe oder Grundsicherungsleistungen im Alter und bei '
                         'Erwerbsminderung (nach dem Zwlften Buch Sozialgesetzbuch - SGB XII), <br/>'
                         '- einem Kinderzuschlag',
                'typ': ('kat', 'ja_ne_un',),
                'name': 'wirt_sit',
                'default': ' '
                },
               ),
     }, 
    {'abschnitt': 'G',
     'title': 'Anregende Institution oder Person',
     'items_data': ({'frage': 'Diese aktuelle Hilfe/Beratung anregende(n) Institution(en) oder Person(en) (Hinweisgeber)',
                     'typ': ('kat', 'aip',),
                     'name': 'aip',
                     'default': ' '
                     },
                    ),
     }, 
    {'abschnitt': 'H',
     'title': 'Familienrichterliche Entscheidungen im Zusammenhang mit der <b>aktuellen Hilfe</b>',
     'items_data': ({'frage': 'Teilweiser oder vollstndiger Entzug der elterlichen Sorge (nach 1666 BGB)',
                     'typ': ('kat', 'ja_nein',),
                     'name': 'ees',
                     'default': '2',
                     'no_empty_option': True,     
                     },
                    {'frage': 'Gerichtliche Anordnung der Beratung (nach 156 Absatz 1 Satz 4 FamFG)',
                     'typ': ('kat', 'ja_nein',),
                     'name': 'va52',
                     'default': '2',
                     'no_empty_option': True,     
                     },
                    {'frage': 'Richterliche Genehmigung fr eine freiheitsentziehende Unterbringung/Manahme (nach 1631b BGB)',
                     'typ': ('kat', 'ja_nein',),
                     'name': 'rgu',
                     'default': '2',
                     'no_empty_option': True,     
                     },
                    ),
     },
    {'abschnitt': 'I',
     'title': 'Hilfe dauert an',
     'items_data': ({'frage': 'Hilfe/Beratung dauert am Jahresende an',
                     'typ': ('kat', 'ja_nein',),
                     'name': 'hda',
                     'default': ' '
                     },
                    ),
     }, 
    {'abschnitt': 'J',


     'title': 'Intensitt der am Jahresende andauernden Hilfe/Beratung',
     'items_data': ('(Nur ausfllen, falls die Hilfe/Beratung am Jahresende andauert)',
                    {'frage': 'Zahl der Beratungskontakte im abgelaufenen Kalenderjahr',
                     'typ': ('int', 1, 999,),
                     'name': 'nbkakt',
                     'default': ' ',
                     'extra': (config.BERATUNGSKONTAKTE and 
                               'Falls leer, wird die aus den Beratungskontakten berechnete Anzahl eingesetzt.' or 
                               'Mindestens 1 Kontakt angeben'),
                     },
                    ),
     }, 
    {'abschnitt': 'K',
     'title': 'Grnde fr die Hilfegewhrung',
     # Die Beispiele fr die Grnde stehen hier, weil im Datenbankschema fr die Tabelle code,
     # wo sie eigentlich stehen sollen, fr das Feld name nur 160 Zeichen vorgesehen sind.
     'gruende_bsp': {'10':"(z.B. Ausfall der Bezugspersonen wegen Krankheit, stationrer Unterbringung, Inhaftierung, Tod; unbegleitet eingereiste Minderjhrige)",
                 '11':"(z.B. soziale, gesundheitliche, wirtschaftliche Probleme)",
                 '12':"(z.B. Vernachlssigung, krperliche, psychische, sexuelle Gewalt in der Familie)",
                 '13':"(z.B. Erziehungsunsicherheit, pdagogische berforderung, unangemessene Verwhnung)",
                 '14':"(z.B. psychische Erkrankung, Suchtverhalten, geistige oder seelische Behinderung)",
                 '15':"(z.B. Partnerkonflikte, Trennung und Scheidung, Umgangs-/Sorgerechtsstreitigkeiten, Eltern-/Stiefeltern-Kind-Konflikte, migrationsbedingte Konfliktlagen)",
                 '16':"(z.B. Gehemmtheit, Isolation, Geschwisterrivalitt, Weglaufen, Aggressivitt, Drogen-/Alkoholkonsum, Delinquenz/Straftat)",
                 '17':"(z.B. Entwicklungsrckstand, ngste, Zwnge, selbst verletzendes Verhalten, suizidale Tendenzen)",
                 '18':"(z.B. Schwierigkeiten mit Leistungsanforderungen, auch durch Hochbegabung, Konzentrationsprobleme ADS, Hyperaktivitt, schulvermeidendes Verhalten, Schulschwnzen)",
                 '19':"",
                 },

     },
    'Nachfolgende Angaben bitte zustzlich beim Ende der Hilfe/Beratung ausfllen.',
    {'abschnitt': 'L',
     'title': 'Ende der Hilfe/Beratung',
     'items_data': ({'frage': 'Monat',
                     'typ': ('int', 1, 12,),
                     'name': 'em',
                     },
                    {'frage': 'Jahr',
                     'typ': ('int', 1970, 2030,),
                     'name': 'ey',
                     },
                    ),
     }, 
    {'abschnitt': 'M',
     'title': 'Betreuungsintensitt der beendeten Hilfe/Beratung',
     'items_data': ({'frage': 'Zahl der Beratungskontakte whrend der gesamten Beratungsdauer',
                     'typ': ('int', 1, 999,),
                     'name': 'nbkges',
                     'default': ' ',
                     'extra': (config.BERATUNGSKONTAKTE and 
                               'Falls leer, wird die aus den Beratungskontakten berechnete Anzahl eingesetzt.' or 
                               'Mindestens 1 Kontakt angeben'),
                     },
                    {'frage': 'Letzter Beratungskontakt liegt mehr als sechs Monate zurck',
                     'typ': ('kat', 'ja_nein',),
                     'name': 'lbk6m',
                     'default': ' '
                     },
                    ),
     },
    {'abschnitt': 'N',
     'title': 'Grund fr die Beendigung der Hilfe/Beratung',
     'items_data': ({'frage': 'Grund fr die Beendigung der Hilfe/Beratung',
                     'typ': ('kat', 'grende',),
                     'name': 'grende',
                     'default': ' '
                     },
                    ),
     }, 
    {'abschnitt': 'O',
     'title': 'Anschlieender Aufenthalt',
     'items_data': ({'frage': 'Anschlieender Aufenthalt',
                     'typ': ('kat', 'auf_ort',),
                     'name': 'aort_nac',
                     'default': ' '
                     },
                    ),
     }, 
    {'abschnitt': 'P',
     'title': 'Unmittelbar nachfolgende Hilfe',
     'items_data': ({'frage': 'Unmittelbar nachfolgende Hilfe',
                     'typ': ('kat', 'unh',),
                     'name': 'unh',
                     'default': ' '
                     },
                    ),
     }, 
    )


class _jgh07(Request.Request, akte_share):
    """Gemeinsame Klasse fr die neue Bundesstatistik"""
    def _formular_neu(self, jgh):
        """
        jghdef-gesteuerter Aufbau der JGH-Maske
        """
        fall = jgh['fall']
        # In die Falldaten ganz oben eine Information einfgen
        # wenn der Fall abgeschlossen ist.
        if not fall or not fall['aktuell']:
            jgh['fallabschluss'] = ("%s" % (fall.getDate('zda')
                                            if fall else "{}.{}".format(jgh['em'], jgh['ey'])))
        else:
            jgh['fallabschluss'] = 'offen'
        # Nur zur Anzeige auf der JGH-Seite.
        # Wohnt_Ausserhalb wird nicht in die Datenbank eingetragen.
        # Bei Export der Daten wird das wieder neu ermittelt.
        jgh['wohnt_ausserhalb'] = get_wohnt_ausserhalb(jgh)
        jgh['beginn'] = "{}.{}".format(jgh['bgm'], jgh['bgy'])
        jgh['berichtseinheitid'] = get_berichtseinheitid(jgh['stz__code'])
        jahr = jgh['jahr']
        jghdef = get_jghdef(jahr, strict=False)
        #print 'JGHSTATISTIK: Today: {} Bundesstatistik Version: {}'.format(today(), jghdef.berichtsjahr)
        rows = jghdef.generate_teile(jgh)
        rows.append(self.modifikation_zulassen(fall, "Jugendhilfestatistik"))
        hidden = (('file', jgh['file']),
                  ('stz', jgh['stz']),
                  ('fallid', jgh['fall_id']),
                  # ('fall_fn', jgh['fall_fn']),
                  ('mitid', jgh['mit_id']),
                  ('jghid', jgh['jghid']),
                  ('jahr', jahr),
                  )
        # kein speichern fr alte Berichtsjahre
        url_klkarte = 'klkarte?akid=%(fall__akte_id)s' % jgh
        speichern_etc = [h.SpeichernZuruecksetzenAbbrechen(
            onclick_abbrechen="go_to_url('{}')".format(url_klkarte)
        ),] if jahr >= 2022 else [
            h.FieldsetInputTable(daten=[[h.String(
                string="Jugendhilfestatistik fr Berichtsjahre vor 2022 nicht vernderbar",
                align='center',
                class_='legendtext',)]])]
        res = h.FormPage(
            title='Bundesstatistik bearbeiten',
            help=False,
            name="jghform",action="klkarte",method="post",
            breadcrumbs = (('Hauptmen', 'menu'),
                           ('Klientenkarte', url_klkarte),
                           ),
            rows=rows + speichern_etc,
            hidden=hidden,
            )
        return res.display()
        
    def _formular(self, jgh):
        """
        Obsolet. Nur noch zum Testen, kann spter dann wegfallen.
        Gesteuert von _fb_data-Struktur, siehe oben.
        Wird aufgerufen mit dem zustzlichen URL-Parameter v=1.

        Bringt Templates und Daten zusammen sowohl fr die Neueinfhrung
        als auch fr das Update.
        """
        # default ist formular_neu mit jghdef
        # v1 nur zum testen
        if not self.v1:
            return self._formular_neu(jgh)
        fall = jgh['fall']
        # In die Falldaten ganz oben eine Information einfgen
        # wenn der Fall abgeschlossen ist.
        if not fall or not fall['aktuell']:
            jgh['abschluss_info_row'] = jgh_abschluss_info_t % {'abschluss_info': (("Der Fall wurde am %s abgeschlossen." % fall.getDate('zda')) if fall else 
                                                                                   "Der Fall ist abgeschlossen.")}
        else:
            jgh['abschluss_info_row'] = ''
        jgh['land_sel'] = self.for_land_kr_einrnr('land')
        jgh['kr_sel'] = self.for_land_kr_einrnr('kr')
        jgh['einrnr_sel'] = self.for_land_kr_einrnr('einrnr')
##         jgh['land_sel'] = ''.join(make_option_list(get_codes('land'), 'first', True))
##         jgh['kr_sel'] = ''.join(make_option_list(get_codes('kr'), 'first', True))
##         jgh['einrnr_sel'] = ''.join(make_option_list(get_codes('einrnr'), 'first', True))
        jgh['gfall_sel'] = ''.join(make_option_list(get_codes('gfall'), 'first', True))
        if jgh.get('lnr'):
            jgh['laufendenr'] = jgh['lnr']
        else:
            jgh['laufendenr'] = 'noch nicht vergeben'
        # wohnung auerhalb
        plz = ort = ags = ''
        ags_plz_ort = get_ags_plz_ort_if_wohnort_ausserhalb(jgh)
        if ags_plz_ort:
            ags, plz, ort = ags_plz_ort
        if ags:
            jgh['ags'] = "(AGS: %s)" % ags
        else:
            jgh['ags'] = ''
        jgh['plz'], jgh['ort'] = plz, ort
        res = []
        res.append(jghhead_t % jgh)
        res.append(jghfalldaten_t % jgh)
        res.append(fb_abschnitt_trenner_t)
        jgh['beginn'] = "{}.{}".format(jgh['bgm'], jgh['bgy'])
        jgh['wohnt_ausserhalb'] = "{} {} {}".format(jgh['plz'], jgh['ort'], jgh['ags'])
        res.append(fb_abschnitt_trenner_t.join(
            [self._abschnitt(a, jgh) for a in _fb_data]))
        res.append(fb_abschnitt_trenner_t)
        res.append(self.modifikation_zulassen(fall, "Jugendhilfestatistik"))
        res.append(jghbuttons_t) 
        res.append(jghfoot_t)
        if fall:
            klkarte_url = 'klkarte?akid=%(akte_id)s' % fall
        else:
            klkarte_url = ''
        return h.Page(
            title='Bundesstatistik bearbeiten',
            help=False,
            breadcrumbs = (('Hauptmen', 'menu'),
                           ('Klientenkarte', klkarte_url),
                           ),
            rows=res
            ).display()

    def _abschnitt(self, data, jgh):
        if isinstance(data, basestring):
            return fb_zwischenueberschrift_t % data
        abschnitt = data.copy()
        if abschnitt['abschnitt'] == 'K':
            # besondere Behandlung
            items = self._items_fuer_K(jgh, abschnitt['gruende_bsp'])
        else:
            items = [self._item(i, jgh) for i in data['items_data']]
        abschnitt['items'] = fb_item_trenner_t.join(items)
        return fb_abschnitt_t % abschnitt

    def _items_fuer_K(self, jgh, gruende_bsp):
        res = []
        res.append(fb_k_header_t)
        codes = get_codes('gruende')
        for c in codes:
            id = c['id']
            code = c['code']
            bsp = gruende_bsp[code]
            name = c['name']
            if bsp:
                frage = "%s<br>%s" % (name, bsp)
            else:
                frage = name
            item = {'frage': frage,
                    'id': id,
                    'ch1': jgh.get('gr1') == id and 'checked' or '',
                    'ch2': jgh.get('gr2') == id and 'checked' or '',
                    'ch3': jgh.get('gr3') == id and 'checked' or '',
                    }
            if code == '19':
                res.append(fb_k_last_item_t % item)
            else:
                res.append(fb_k_item_t % item)
        return res
    def _item(self, data, jgh):
        if isinstance(data, basestring):
            return fb_zwischenueberschrift_t % data
        assert isinstance(data, dict)
        item = data.copy()
        label = item['typ'][0]
        if label == 'kat':
            kat_code = item['typ'][1]
            name = item['name'] # Feldname
            default = jgh.get(name) # vorhandener Wert in jgh
            if default == None: # kein vorhandener Wert
                default = item.get('default') # default aus _fb_data einsetzen
                if default and default != ' ': # nachschlagen, falls default ein echter code ist
                    default = cc(kat_code, default)
            codes = get_codes(kat_code)
            #item['options'] = mkselstr(fb_option_t, codes, 'id', default)
            no_empty_option = item.get('no_empty_option')
            item['options'] = ''.join(make_option_list(codes, default, no_empty_option))
            maxl = max([len(c['name']) for c in codes])
            if maxl < 60:
                w = "%sem" % max(maxl, 3)
            else:
                w = "100%"
            #item['width'] = w
            item['width'] = "100%"
            if name == 'hda':
                item['onChange'] = """onChange="del_anzahl_kontakte('%s', '%s')" """ % \
                                   (cn('ja_nein', 'ja'), cn('ja_nein', 'nein'))
            else:
                item['onChange'] = ''
            return fb_select_one_item_t % item
        elif label == 'int':
            minl, maxl = item['typ'][1:3]
            item['size'] = len(str(maxl))
            name = item['name']
            default = jgh.get(name)
            if default == None:
                default = item.get('default')
                if default == None:
                    default = ''
            item['value'] = default
            if item.get('readonly'):
                item['ro'] = 'readonly'
            else:
                item['ro'] = ''
            extra = item.get('extra')
            if extra:
                item['tip'] = tip_t % {'tip': extra}
            else:
                item['tip'] = ''
            return fb_int_item_t % item
        elif label == 'checkbox':
            name = item['name']
            value = jgh.get(name)
            item['value'] = '1'
            item['checked'] = (value == '1' and 'checked' or '')
            return fb_checkbox_item_t % item
        else:
            return ''

class updjgh07(_jgh07):
    """Jugendhilfestatistik ndern. (Tabelle: Jugendhilfestatistik)"""
    
    permissions = Request.STAT_PERM
    
    def processForm(self, REQUEST, RESPONSE):
        self.v1 = self.form.get('v1')
        # Pro Fall kann es nur 1 Jugendhilfestatistik geben.
        fallid = self.form.get('fallid')
        jahr_change = self.form.get('jahr_change')
        jahr = self.form.get('jahr')
        if not jahr_change and jahr and int(jahr) < 2007:
            raise EE("Jugendhilfestatistik von vor 2007 kann nicht mehr angezeigt werden.")
        id = self.form.get('jghid')
        fall = None
        if id:
            jgh = Jugendhilfestatistik2007(id)
            fall = jgh['fall']
        elif fallid:
            fall = Fall(fallid)
            jgh = fall['jgh']
            if not jgh:
                raise EE('Noch keine Bundesstatistik f&uuml;r den Fall vorhanden.')
        else:
            self.last_error_message = "Keine Bundesstatistik-ID erhalten"
            return self.EBKuSError(REQUEST, RESPONSE)
        jgh['jghid'] = jgh['id']
        jgh['file'] = 'updjgh07'
        if jahr_change:
            jahr_change = int(jahr_change)
            if jahr_change < jgh['fall__bgy'] or jahr_change > today().year:
                raise EE('Fehler im Berichtsjahr')
            # auergewhnlich das Update gleich hier zu machen,
            # aber ich sehe jetzt keine andere gute Lsung.
            jgh.update(dict(jahr=jahr_change))
        return self._formular(jgh)
        
class jgh07neu(_jgh07):
    """Neue Jugendhilfestatistik eintragen. (Tabelle: Jugendhilfestatistik.)

    Das Folgenden gilt nur fr die Steuerung ber die neu einfhrte 
    Bundesstatistikdefinition (ab Version 4.13).

    Funktionsvoraussetzung fr EBKuS:
    - es muss immer eine Bundesstatistikdefinition fr jedes Jahr seit 2022 bis
    zum Vorjahr des aktuellen Jahrs vorhanden sein.
    - im Laufe des aktuellen Jahres kommt die Bundesstatistikdefinition fr dieses
    aktuelle Jahr hinzu, allersptestens am Ende des aktuellen Jahres ist sie da,
    damit die Statistiken fr das Jahr gemacht werden knnen.
    - Bundesstatistikdefinitionen fr die Zukunft werden grundstzlich ignoriert.

    Auswahl der Bundesstatistikdefinition fr das JGH-Formular in Abhngigkeit 
    von dem Berichtsjahr:
    
    JGH vorhanden, also Updates:
    - Updates der Statistik fr Berichtsjahre vor 2022 werden nicht zugelassen.
    - Man kann also davon ausgehen, dass immer eine Bundesstatistikdefinition fr das
      gewnschte Jahr da ist.
    - Sollte das doch mal nicht der Fall sein, wird die neueste genommen, also die
      aus dem Vorjahr.
    - Problem: Altflle mit 2007 < Berichtsjahr < 2022.
      Anzeige mit Bundesstatistikdefinition 2022.
      Kein Speichernbutton.
      Angebot auf nderung des Berichtsjahrs auf mindestens 2022.
      (Noch ltere Flle werden auch nicht mehr angezeigt!)

    Kein JGH vorhanden, Neuanlage:
    - Grundstzlich nur fr aktuelles Jahr und Vorhahr (ganz evt. fr Vorvorjahr)

    Auswahl der Bundesstatistikdefinition fr die Validierung in ebupd:
    
    - Immer das Berichtsjahr. Nur mglich ab Berichtsjahr 2022. Notfalls
      Bundesstatistikdefinition vom Vorjahr nehmen.
    
    Bestimmung des Jahres

    ber das Jahr wird auch die jghdef ausgewhlt.

    Ist nur bei aktuellem Fall mglich. Normalerweise ist das ein Fall,
    der im aktuellen Jahr begann oder im Vorjahr begann und andauert, 
    sonst gbe schon eine Statistik fr das Vorjahr.

    Kann es sein, dass eine Statistik neu angelegt wird fr das Vorvorjahr?
    Fr meine Tests ist das sinnvoll, aber in der Praxis eigentlich nicht.
    Gengt es, wenn fr einen neuen Fall das aktuelle Jahr als Berichtsjahr
    ausgewhlt wird und zwei Vorjahre zur Auswahl gestellt werden? Das Vorvorjahr
    drfte eigentlich nie gebraucht werden. Eigentlich msste das Vorjahr gengen.

    Auswahl der Bundesstatistikdefinition fr den JGH-Export:
    - Immer nur genau das Berichtsjahr.

    Auswahl der Bundesstatistikdefinition fr die Statistikauswertung:
    - Es wird keine Statistikdefinition ausgewhlt, es gengen die Daten
    in der Datenbank. Da keine Items und keine Codes gelscht werden, kann
    die Statistik immer angezeigt werden. Manchmal ergeben sich Abweichungen
    fr frhere Jahre, die werden aber ignoriert.

    Falls fr Testzwecke today() auf ein frheres Datum zurckgesetzt wurde:
    - schwierig fr alte Datenbanken, aber ab 2022 sollte das wieder ganz gut gehen
    """
    permissions = Request.STAT_PERM
    def processForm(self, REQUEST, RESPONSE):
        self.v1 = self.form.get('v1')
        fallid = self.form.get('fallid')
        jghid = self.form.get('jghid')
        jahr_str = self.form.get('jahr_change')
        if jahr_str:
            jahr = int(jahr_str)
        else:
            jahr = today().year
        if not fallid:
            raise EE('Bundesstatistik nur f&uuml;r einen g&uuml;ltigen Fall')
        fall = Fall(fallid)
        ex_jgh = fall['jgh']
        if ex_jgh:
            raise EE('Bundesstatistik bereits vorhanden.')
        jgh = Jugendhilfestatistik2007()
        if jghid and get_object(jghid, Jugendhilfestatistik2007) is None:
            # Formular wurde schon mit neuer id aufgerufen,
            # aber nicht gespeichert wg. Berichtsjahrwechsel
            jgh['jghid'] = jghid
        else:
            jgh['jghid'] = Jugendhilfestatistik2007().getNewId()
        jgh['stz'] = self.stelle['id']
        jgh['file'] = 'jgh07einf'
        jgh['mit_id'] = fall['zustaendig__mit_id']
        #jgh.setDate('bg', fall.getDate('bg'))
        jgh.setDate('bg', fall['leistungsbeginn'])
        jgh['fall_fn'] = fall['fn']
        jgh['fall_id'] = fall['id']
        jgh['jahr'] = jahr
        dmy = [int(e) for e in fall['akte__gb'].split('.')]
        dmy.reverse()
        geburtsdatum = ebapi.Date(*dmy)
        jgh['gey'] = geburtsdatum.year
        jgh['gem'] = geburtsdatum.month
        geschlecht = fall['akte__gs']
        if not geschlecht:
            raise EE("Bitte zuerst das Geschlecht in die Klientenkarte eintragen!")
        jgh['gs'] = geschlecht
        assert isinstance(jgh['gs'], (int, long))
        if config.BERATUNGSKONTAKTE:
            from ebkus.html.beratungskontakt import get_jgh_kontakte
            jgh['nbkakt'], jgh['nbkges'] = get_jgh_kontakte(fall, jahr)
        # for name, kat in (('Bundesland', 'land'), ('Kreis', 'kr'), ('Einrichtungsnummer', 'einrnr')):
        #     code = self.get_land_kr_einrnr(self, kat)
        #     if code is None:
        #         raise EE("'{}' fr Ihre Stelle ist nicht richtig konfiguriert".format(name))
        #     jgh[kat] = code['id']
        return self._formular(jgh)
        
from ebkus.html.jgh_wohnt_ausserhalb import wohnt_ausserhalb
from ebkus.html.strkat import get_strasse

def _get_adress_data(jgh):
    """Holt Adressdaten ergnzt um Info aus dem Strassenkatalog falls vorhanden.
    Zur Zeit nur aus Akte, also Klient selber.
    Evt. auch aus Bezugsperson, aber nicht klar wie.
    Klient knnte allein wohnen, o.

    Es werden nur Adressen herangezogen, wenn es eine 5-stellige PLZ und 
    einen Eintrag fr den Ort gibt.
    """
    # def daten_ausreichend(data):
    #     if not re.match(r'^\d{5,5}$', data['plz']): # Nur korrekt geformte plz verwenden.
    #         data['plz'] = ''
    #         if not data['plz'] or not data['ort']: # Ohne korrekte plz und ort kann 'wohnt ausserhalb' nicht besttigt werden.
    #             return False
    def data_from_obj(obj):
        data = dict(
            hsnr=obj.get('hsnr', ''), 
            str=obj.get('str', ''), 
            plz=obj.get('plz', ''),
            ort=obj.get('ort', ''),
            ortsteil=obj.get('ortsteil', ''),
            samtgemeinde=obj.get('samtgemeinde', ''),
            bezirk=obj.get('bezirk', ''),
            plraum=obj.get('plraum', ''),
            )
        return data
        # update nimmt keine keyword-Argumente in python 2.3:
        # data.update(hsnr=obj['hsnr'], str=obj['str'], plz=obj['plz'], ort=obj['ort'], plraum=obj['plraum'])
        # data.update(d)

        # d = (hsnr=obj['hsnr'], str=obj['str'], plz=obj['plz'], ort=obj['ort'], plraum=obj['plraum'])
        # for k in data.keys():
        #     v = obj.get(k)
        #     if v:
        #         data[k] = v
    try:
        akte = jgh['fall__akte']
    except AttributeError:
        akte = None
    if akte:
        adressquelle = akte
        data = data_from_obj(adressquelle)
        # passiert alles spter:
        # if not daten_ausreichend(data):
        #     return None
            # for art in ('Mutter', 'Vater',):
            #     for bp in akte['bezugspersonen']:
            #         if bp['verw__name'] == art:
        if config.STRASSENKATALOG:
            strasse = get_strasse(adressquelle)
            data.update(strasse)
        data['fall_fn'] = jgh['fall_fn']
        return data
    return None

def get_ags_plz_ort_if_wohnort_ausserhalb(jgh, spec=None):
    data = _get_adress_data(jgh)
    if data and wohnt_ausserhalb(data, spec=spec):
        plz, ort = data['plz'], data['ort']
        ags = get_gemeindeschluessel(ort, plz)
        # print 'Spec:', spec
        # print 'Data:', data['plz'], data
        # print 'Wohnt ausserhalb True' 
        # print 'Ags: ', plz, ort
        return ags, plz, ort
    else:
        # print 'Spec:', spec
        # print 'Data:', data['plz'], data
        # print 'Wohnt ausserhalb False' 
        return None
def get_wohnt_ausserhalb(jgh):
    """
    Liefert String zur Anzeige auf der JGH-Seite, falls ermittelt
    wurde, dass der Klient ausserhalb wohnt.
    (Siehe jgh_wohnt_ausserhalb.py, dort wohnt_ausserhalb()).
    Im String steht der Gemeindeschlssel, falls dieser bekannt ist
    (Siehe gemeindeschluessel.py), sonst PLZ und Ort.
    Falls nicht festgestellt wurde, dass der Klient auerhalb wohnt,
    wird ein leerer String zurckgegeben.
    """
    plz = ort = ags = res = ''
    ags_plz_ort = get_ags_plz_ort_if_wohnort_ausserhalb(jgh)
    if ags_plz_ort:
        ags, plz, ort = ags_plz_ort
        ags_str = "  (amtlicher Gemeindeschlssel: {})".format(ags) if ags else ''
        return "{} {}{}".format(plz, ort, ags_str)
    else:
        return ''
    
class jgh_wohnt_ausserhalb_check(Request.Request, akte_share):
    """Liefert Liste von Adressen & ob wohnt_ausserhalb detektiert wird mit aktueller Konfiguration.
    Wird nur bentigt zur Prfung der Konfiguration. 
    """
    permissions = Request.ADMIN_PERM

    def get_daten(self, jahr=None, spec=None):
        """Adressdaten fr alle Bundesstatistikdatenstze fr jahr.
        Liefert Liste von dicts mit Feldern:
        felder = 'str','hsnr','plz','ort',
                 'ortsteil','samtgemeinde','bezirk' # je nach config.STRASSENSUCHE
                 'plz_ausser', 'ort_ausser'
        damit man prfen kann, fr welche Adressen "wohnt auerhalb" generiert wird.
        """
        if jahr:
            jahr = int(jahr)
        else:
            jahr = today().year - 1
        if spec is None:
            spec = config.WOHNT_NICHT_AUSSERHALB
        where = 'jahr = %s' % jahr
        jghlist = Jugendhilfestatistik2007List(where=where)
        def get_data(jgh):
            data = _get_adress_data(jgh)
            if not data:
                return None
            res = get_ags_plz_ort_if_wohnort_ausserhalb(jgh, spec=spec)
            if not res:
                res = '','',''
            data['ags_ausser'], data['plz_ausser'], data['ort_ausser']  = res
            return data
        # TODO nur zum testen die flle ohne ort ausgesperrt
        daten = [data for data in [get_data(jgh) for jgh in jghlist] if data]
        #daten = [data for data in [get_data(jgh) for jgh in jghlist] if data and data['ort']]
        return daten

    def get_felder_und_daten_zeilen(self, jahr, spec):
        """Liefert Liste von Listen von StringItems fr FieldsetDataTable"""
        felder = 'fall_fn', 'str','hsnr','plz','ort'
        zusatzfelder = config.STRASSENSUCHE.split()
        if 'ort' in zusatzfelder:
            zusatzfelder.remove('ort')
        felder += tuple(zusatzfelder)
        if 'plraum' not in felder:
            felder += ('plraum',)
        felder += ('plz_ausser', 'ort_ausser', 'ags_ausser')
        daten_zeilen = []
        for data in self.get_daten(jahr, spec):
            res = []
            for f in felder:
                res.append(h.String(string=data[f]))
            daten_zeilen.append(res)
        return felder, daten_zeilen

    def post(self, REQUEST, RESPONSE):
        jahr = self.form.get('jahr')
        wohnt_nicht_ausserhalb_test = self.form.get(
            'wohnt_nicht_ausserhalb_test',
            config.WOHNT_NICHT_AUSSERHALB)
        
        uebernehmen = self.form.get('uebernehmen')
        if uebernehmen:
            updkonfig({'konfig_name':'wohnt_nicht_ausserhalb',
                       'konfig_value': wohnt_nicht_ausserhalb_test})
        wohnt_nicht_ausserhalb_test = urllib.quote(wohnt_nicht_ausserhalb_test)
        RESPONSE.redirect('jgh_wohnt_ausserhalb_check?jahr={}&'
                      'wohnt_nicht_ausserhalb_test={}'.format(jahr, wohnt_nicht_ausserhalb_test), 
                      status=303)
    def get(self, REQUEST, RESPONSE):
        jahr = self.form.get('jahr')
        if jahr:
            jahr = int(jahr)
        else:
            jahr = today().year - 1
        wohnt_nicht_ausserhalb_test = self.form.get(
            'wohnt_nicht_ausserhalb_test',
            config.WOHNT_NICHT_AUSSERHALB)
        konfig_validate(config, 'wohnt_nicht_ausserhalb', 
                        wohnt_nicht_ausserhalb_test, fachlich_only=True)
        felder, daten_zeilen = self.get_felder_und_daten_zeilen(jahr, wohnt_nicht_ausserhalb_test)
        adress_table = h.FieldsetDataTable(
            legend="Adressen zur Prfung der Konfiguration ({})".format(len(daten_zeilen)),
            tip="Die Felder plz_ausser und ort_ausser enthalten Werte, wenn der Wohnort des Klienten als auerhalb liegend erkannt wird. "
            "Zustzlich ist das Feld ags_ausser mit dem amtlichen Gemeindeschlssel belegt, wenn dieser gefunden wurde. "
            "Dementsprechend wird das Feld 'Wohnort auerhalb' in der Bundesstatistik automatisch belegt. "
            "Dokumentation fr die Konfigurationsvariable 'wohnt_nicht_ausserhalb' findet sich unter 'Einstellungen' im Administratormen.",
            headers=felder,
            header_class='sortable',
            daten=daten_zeilen,
            )
        konfig_table = h.FieldsetFormInputTable(
            legend="Konfiguration von <code>wohnt_nicht_ausserhalb</code> prfen",
            name='wohnt_nicht_ausserhalb',
            action='jgh_wohnt_ausserhalb_check',
            method='post',
            hidden = (('welche', 'fortdauernd'),),
            daten=[
                [h.SelectItem(label='Jahr',
                                 name='jahr',
                                 class_='listbox60',
                                 tip='Jahr whlen, fr das Adressen geprft werden',
                                 options=self.for_jahre(sel=jahr),
                                 ),
                 h.TextItem(label="Konfigurierter Wert von 'wohnt_nicht_ausserhalb'",
                            class_='textbox310',
                            value=config.WOHNT_NICHT_AUSSERHALB,
                            tip="Der unter Einstellungen konfigurierte Wert fr 'wohnt_nicht_ausserhalb', siehe auch die Dokumentation dort",
                            readonly=True,
                            ),
                 ],[h.CheckItem(label='Testwert in die Konfiguration bernehmen',
                                name='uebernehmen',
                                value=1,
                                checked=False,
                                tip='Hkchen machen, um den aktuellen Testwert dauerhaft in die Konfiguration zu bernehmen',
                                ),
                    h.TextItem(label="Zu testender Wert von 'wohnt_nicht_ausserhalb'",
                               class_='textbox310',
                               name='wohnt_nicht_ausserhalb_test',
                               value=wohnt_nicht_ausserhalb_test,
                               tip="Hier einen Wert fr 'wohnt_nicht_ausserhalb_test' eingeben und Ergebnisse in der Tabelle berprfen"
                            ),
                    ],[h.Button(value="Anzeigen",
                                name='submitop',
                                tip="Tabelle mit Prfung 'wohnt_nicht_ausserhalb_test' anzeigen",
                                type='submit',
                                n_col=4,
                                ),
                       ]
                 ],
        )
        res = h.Page(
            title='Adressen zur Prfung der Bundesstatistik',
            rows=(self.get_hauptmenu(),
                  konfig_table,
                  adress_table,
                  ),
            )
        return res.display()

    
